# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from knack.util import todict
from knack import events
from knack.util import CLIError
from azure.cli.core.commands.events import EVENT_INVOKER_PRE_LOAD_ARGUMENTS

logger = get_logger(__name__)


def register_global_query_examples_argument(cli_ctx):
    """Register --query-examples argument, and register handler."""

    def handle_example_parameter(cli, **kwargs):  # pylint: disable=unused-argument
        args = kwargs['args']
        # `query_examples` has been registered and `--query-examples` appers in the command args
        if hasattr(args, '_query_examples') and args._query_examples is not None:  # pylint: disable=protected-access
            if cli_ctx.invocation.data['query_active']:
                raise CLIError('You should not use --query and --query-examples together.')
            # change the default output format to table
            if cli_ctx.invocation.data['output'] == 'json':
                cli_ctx.invocation.data['output'] = 'table'

            def analyze_output(cli_ctx, **kwargs):
                tree_builder = QueryTreeBuilder(cli.config)
                tree_builder.build(kwargs['event_data']['result'])
                # replace the result with query examples
                kwargs['event_data']['result'] = tree_builder.generate_examples(
                    args._query_examples, cli_ctx.invocation.data['output'])  # pylint: disable=protected-access
                cli_ctx.unregister_event(
                    events.EVENT_INVOKER_FILTER_RESULT, analyze_output)

            cli_ctx.register_event(
                events.EVENT_INVOKER_FILTER_RESULT, analyze_output)
            cli_ctx.invocation.data['query_active'] = True

    def register_query_examples(cli, **kwargs):
        from knack.experimental import ExperimentalItem
        experimental_info = ExperimentalItem(cli.local_context.cli_ctx,
                                             object_type='parameter', target='_query_examples')
        default_kwargs = {
            'help': 'Recommend JMESPath string for you. You can copy one of the query '
                    'and paste it after --query parameter within double quotation marks '
                    'to see the results. You can add one or more positional keywords so '
                    'that we can give suggestions based on these key words.',
            'arg_group': 'Global',
            'is_experimental': True,
            'nargs': '*',
            'experimental_info': experimental_info
        }

        allow_list = cli.config.get('query', 'allow_list', "list,show").split(',')
        allow_list = [s.strip() for s in allow_list if s.strip()]  # remove empty string
        # when allow_list="", do nothing
        if not allow_list:
            return

        commands_loader = kwargs.get('commands_loader')
        cmd_tbl = commands_loader.command_table
        for cmd_name, cmd in cmd_tbl.items():
            if any(cmd_name.endswith(suffix) for suffix in allow_list):
                cmd.add_argument('_query_examples', *
                                 ['--query-examples'], **default_kwargs)

    cli_ctx.register_event(
        EVENT_INVOKER_PRE_LOAD_ARGUMENTS, register_query_examples
    )
    cli_ctx.register_event(
        events.EVENT_INVOKER_POST_PARSE_ARGS, handle_example_parameter)


class QueryExample:
    def __init__(self, query_str, help_str="", max_length=None):
        self._query_str = query_str
        self._help_str = help_str
        self._examples_len = max_length
        self._help_len = max_length
        self.escape_char()

    def set_max_length(self, examples_len, help_len):
        self._examples_len = examples_len
        self._help_len = help_len

    def escape_char(self):
        """Escape special characters in JMESPath"""
        # escape backtick
        self._query_str = self._query_str.replace('`', '\\`')

    def _asdict(self):
        query_str = self._query_str
        if self._examples_len and len(query_str) > self._examples_len:
            query_str = query_str[:self._examples_len] + '...'
        help_str = self._help_str
        if self._help_len and len(help_str) > self._help_len:
            help_str = help_str[:self._help_len] + '...'
        return {"query string": query_str, "help": help_str}

    def __str__(self):
        return "{}\t{}".format(self._query_str, self._help_str)


class QueryTreeNode:
    def __init__(self, name, parent, is_array):
        self.is_dummy = False
        self._name = name
        self._parent = parent
        self._is_array = is_array  # inside an JSON array
        self._data = None
        self._child = []  # list of child node

    def add_child(self, child_node):
        if child_node:
            self._child.append(child_node)

    def update_node_data(self, data):
        self._data = data

    def _get_one_data(self):
        """Try to get not None data."""
        if not self._data:
            return None
        for value in self._data:
            if value:
                return value
        return None

    def get_help_str(self, help_type):
        """Return help string based on help_type."""
        help_table = {
            'contains': 'Show the {} field that contains given string.'.format(self._name),
            'filter': 'Show the resources that satisfy the condition.',
            'select': 'Show the value of {} field.'.format(self._name),
        }
        return help_table.get(help_type, '')

    def get_trace_to_array(self, inner_trace):
        """
        Get trace to the nearest array. Return None if no array found. This function return two values,
        inner trace and outer trace. The inner trace is the trace inside the brackets and the outer
        trace is the trace outside the brackets.
        """
        if self._is_array:
            if self._parent:
                if self._parent.is_dummy and not self._parent._is_array:  # pylint: disable=protected-access
                    outer_trace = self._name
                else:
                    outer_trace = '{}.{}'.format(self._parent.get_trace_to_root(), self._name)
            else:
                outer_trace = self._name
            return outer_trace, inner_trace

        # under a dict
        if self._parent:
            if inner_trace:
                current_trace = self._name + '.' + inner_trace
            else:
                current_trace = self._name
            return self._parent.get_trace_to_array(current_trace)

        # no array found until root node
        return None, None

    def get_trace_to_root(self):
        """Return the trace string(aka, select string in JMESPath) to the root node."""
        if self._parent:
            if self._parent.is_dummy and not self._parent._is_array:  # pylint: disable=protected-access
                trace_str = self._name
            else:
                trace_str = '{}.{}'.format(self._parent.get_trace_to_root(), self._name)
        else:
            trace_str = self._name
        if self._is_array:
            trace_str += '[]'
        return trace_str

    def get_filter_str(self):
        """Return the kind of JMESPath that filters the results with some conditions."""
        outer_trace, inner_trace = self.get_trace_to_array('')
        if outer_trace is None or inner_trace is None:
            return None
        value = self._get_one_data()
        if not value:
            return None
        filter_str = "{}[?{}=='{}']".format(outer_trace, inner_trace, value)
        return filter_str

    def get_contains_str(self):
        """Return the JMESPath that filter the results with a contains function."""
        outer_trace, inner_trace = self.get_trace_to_array('')
        if outer_trace is None or inner_trace is None:
            return None
        value = self._get_one_data()
        if not value:
            return None
        contains_str = "{0}[?contains(@.{1}, 'something')==`true`].{1}".format(outer_trace, inner_trace)
        return contains_str

    def get_examples(self):
        """The entry function for recommend examples for current node."""
        ans = []
        select_string = self.get_trace_to_root()
        ans.append(QueryExample(select_string, self.get_help_str('select')))
        filter_str = self.get_filter_str()
        if filter_str:
            ans.append(QueryExample(filter_str, self.get_help_str('filter')))
        contains_str = self.get_contains_str()
        if contains_str:
            ans.append(QueryExample(contains_str, self.get_help_str('contains')))
        return ans


class QueryTreeBuilder:
    """Parse entry. Generate parse tree from json. And then give examples."""

    def __init__(self, config):
        self._root = None  # dummy root node
        self._all_nodes = {}
        self._config = {}
        self.update_config(config)

    def build(self, data):
        """Build a query tree with a given json file."""
        self._root = self._parse('', [data], None)
        if self._root:
            self._root.is_dummy = True

    def generate_examples(self, keywords_list, output_format):
        """Generate JMESPath query string examples based on keyword_list."""
        examples = []
        match_list = self._get_matched_nodes(keywords_list)
        for node_name in match_list:
            if node_name:  # skip root node
                for node in self._all_nodes.get(node_name):
                    examples.extend(node.get_examples())
        if self._config['max_examples'] >= 0:
            examples = examples[:self._config['max_examples']]
        if output_format == 'table':
            for item in examples:
                item.set_max_length(self._config['examples_len'], self._config['help_len'])
        return todict(examples)

    def update_config(self, config):
        self._config['examples_len'] = int(config.get('query', 'examples_len', '80'))
        self._config['help_len'] = int(config.get('query', 'help_len', '80'))
        self._config['max_examples'] = int(config.get('query', 'max_examples', '10'))

    def _get_matched_nodes(self, keywords_list):
        """
        Return a list of keys that match the user provided keywords.
        Return all available keys if no keyword is provided.
        """
        def name_match(pattern, line):
            return pattern.lower() in line.lower()

        match_list = []
        # return all available key if no keyword is provided
        if not keywords_list:
            return self._all_nodes.keys()

        for pattern in keywords_list:
            for node_name in self._all_nodes:
                if node_name not in match_list and name_match(pattern, node_name):
                    match_list.append(node_name)
        return match_list

    def _parse(self, name, data, parent, is_array=False):
        """do parse for a single node

        :param str name:
            Name of the node.
        :param list data:
            All data in the json with the same depth and the same name.
            This field must be a list.
        :param TreeNode parent:
            The parent node of current node. None if this is the root node.
        :param bool is_array:
            True if the value of this JSON node is an array.
        """
        if not data:
            return None
        if all(isinstance(d, list) for d in data):
            node = self._parse_list(name, data, parent)
        elif all(isinstance(d, dict) for d in data):
            node = self._parse_dict(name, data, parent, is_array)
        elif any(isinstance(d, (dict, list)) for d in data):
            node = None  # inhomogeneous type
        else:
            node = self._parse_leaf(name, data, parent, is_array)
        return node

    def _parse_list(self, name, data, parent):
        flatten_data = []
        # flatten list of list to list of data
        for d in data:
            flatten_data.extend(d)
        if not flatten_data:
            return None
        node = self._parse(name, flatten_data, parent, is_array=True)
        return node

    def _parse_leaf(self, name, data, parent, is_array):
        node = QueryTreeNode(name, parent, is_array)
        node.update_node_data(data)
        self._record_node(name, node)
        return node

    def _parse_dict(self, name, data, parent, is_array):
        node = QueryTreeNode(name, parent, is_array)
        all_keys = self._get_all_keys(data)
        for key in all_keys:
            values = self._get_not_none_values(data, key)
            if not values:  # all values are None
                continue
            child_node = self._parse(key, values, node, is_array=False)
            node.add_child(child_node)
        self._record_node(name, node)
        return node

    def _get_all_keys(self, data):  # pylint: disable=no-self-use
        """Get all keys in a list of dict"""
        return set().union(*(d.keys() for d in data))

    def _get_not_none_values(self, data, key):  # pylint: disable=no-self-use
        """Get all not None values in a list of dict"""
        return [d.get(key) for d in data if d.get(key, None) is not None]

    def _record_node(self, name, node):
        """Add name and node to `self._all_nodes`"""
        if not node:
            return
        if name in self._all_nodes:
            self._all_nodes[name].append(node)
        else:
            self._all_nodes[name] = [node]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import logging
import multiprocessing
import subprocess
import sys


def init(root):
    parser = root.add_parser('commands', help='Run all the commands with "-h" parameter.')
    parser.add_argument('--prefix', dest='prefix',
                        help='Select the commands with the given prefix. If omit select all the commands.')
    parser.add_argument('--list-only', dest='list_only', action='store_true',
                        help='List the commands\' information instead of execute them in separate processes.')
    parser.add_argument('--details', dest='details', action='store_true',
                        help='List the commands\' details. Use with --list-only.')
    parser.set_defaults(func=run_commands)
    parser.set_defaults(func=run_commands)


def run_commands(args):
    from knack.cli import CLI
    from azure.cli.core import MainCommandsLoader, logger
    from azure.cli.core._config import GLOBAL_CONFIG_DIR, ENV_VAR_PREFIX
    from azure.cli.core.cloud import get_active_cloud

    class MockCLI(CLI):
        def __init__(self):
            super(MockCLI, self).__init__(
                cli_name='mock_cli',
                config_dir=GLOBAL_CONFIG_DIR,
                config_env_var_prefix=ENV_VAR_PREFIX)
            self.cloud = get_active_cloud(self)

    logger.addHandler(logging.NullHandler())

    loader = MainCommandsLoader(MockCLI())
    table = loader.load_command_table('')

    if args.prefix:
        table = dict((each, table[each]) for each in table if each.startswith(args.prefix))

    if args.list_only:
        for each in sorted(table):
            if args.details:
                print_command_info(table[each])
            else:
                print(each)
        sys.exit(0)

    commands = ['az {} --help'.format(each) for each in table]
    results = []

    pool = multiprocessing.Pool(multiprocessing.cpu_count())
    for i, res in enumerate(pool.imap_unordered(run_single_command, commands, 10), 1):
        sys.stderr.write('{:.2f}% \n'.format(float(i) * 100 / len(commands)))
        sys.stderr.flush()
        results.append(res)
    pool.close()
    pool.join()

    if not all(results):
        sys.stderr.write('Error running --help on commands.\n')
        sys.exit(1)

    print('Done')


def print_command_info(command):
    from tabulate import tabulate

    command.load_arguments()

    print('\n{}\n'.format(command.name.upper()))

    attributes = []
    for attr in dir(command):
        if attr.startswith('__'):
            continue

        value = getattr(command, attr)
        if isinstance(value, dict):
            attributes.append({'name': '{}'.format(attr), 'value': ''})
            for k in value:
                attributes.append({'name': '...{}'.format(str(k)), 'value': value[k]})
        else:
            value = str(value)
            attributes.append({'name': attr, 'value': value})

    print(tabulate(attributes, tablefmt='plain'))
    print('<<<<')


def run_single_command(command):
    execution_error = None
    for i in range(3):  # retry the command 3 times
        try:
            subprocess.check_output(command.split(), stderr=subprocess.STDOUT, universal_newlines=True)
            return True
        except subprocess.CalledProcessError as error:
            execution_error = error
            continue

    if execution_error:
        sys.stderr.write('Failed {} -> \n{} / {} \n'.format(command, execution_error.output, str(execution_error)))

    return False



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure_devtools.scenario_tests import RecordingProcessor


class TimeSpanProcessor(RecordingProcessor):
    def __init__(self, replacement):
        self._replacement = replacement

    def process_request(self, request):
        request.uri = self._replace_timespan(request.uri)

        return request

    def _replace_timespan(self, val):
        import re
        # subscription presents in all api call
        retval = re.sub('timespan=((.*?)(&|$))',
                        r'timespan={}\3'.format(self._replacement),
                        val,
                        flags=re.IGNORECASE)
        return retval



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from datetime import datetime, timedelta
import unittest

from azure.cli.testsdk import ScenarioTest, JMESPathCheckExists, ResourceGroupPreparer, \
    StorageAccountPreparer, record_only
from .preparers import VaultPreparer, FileSharePreparer, AFSPolicyPreparer, AFSItemPreparer, \
    AFSRPPreparer, FilePreparer

subscription_id = "da364f0f-307b-41c9-9d47-b7413ec45535"
unprotected_afs = "clitestafs"
protected_afs1 = "sarathtestafs"
protected_afs2 = "afstest1"
target_afs = "afstest2"
storage_account = "sarathsa"
resource_group_name = "sarath-rg"
vault_name = "sarath-vault"


class BackupTests(ScenarioTest, unittest.TestCase):
    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    def test_afs_backup_scenario(self, resource_group, vault_name, storage_account, afs_name, policy_name):
        self.kwargs.update({
            'vault': vault_name,
            'item': afs_name,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'policy': policy_name
        })
        self.cmd('backup protection enable-for-azurefileshare -g {rg} -v {vault} --storage-account {container} --azure-file-share {item} -p {policy}')

        self.kwargs['retain_date'] = (datetime.utcnow() + timedelta(days=30)).strftime('%d-%m-%Y')

        self.kwargs['job'] = self.cmd('backup protection backup-now -g {rg} -v {vault} -c {container} -i {item} --retain-until {retain_date} --backup-management-type AzureStorage --query name').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        self.kwargs['rp'] = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureStorage --query [0].name').get_output_in_json()

        # Trigger Restore
        self.kwargs['job'] = self.cmd('backup restore restore-azurefileshare -g {rg} -v {vault} -c {container} -i {item} -r {rp} --resolve-conflict Overwrite --restore-mode OriginalLocation --query name').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        # Disable Protection
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item} --backup-management-type AzureStorage --delete-backup-data true --yes')
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    @AFSItemPreparer()
    @StorageAccountPreparer(location="southeastasia", parameter_name="sa2")
    @FileSharePreparer(storage_account_parameter_name="sa2", parameter_name="afs2")
    @AFSItemPreparer(afs_parameter_name="afs2", storage_account_parameter_name="sa2")
    def test_afs_backup_container(self, resource_group, vault_name, storage_account, sa2, afs_name, afs2):
        self.kwargs.update({
            'vault': vault_name,
            'sa1': storage_account,
            'sa2': sa2,
            'rg': resource_group,
            'type': "AzureStorage",
            'item1': afs_name,
            'item2': afs2
        })
        container_json = self.cmd('backup container show -n {sa2} -v {vault} -g {rg} --backup-management-type {type}', checks=[
            self.check('properties.friendlyName', '{sa2}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.kwargs['container_name'] = container_json['name']

        self.cmd('backup container show -n {container_name} -v {vault} -g {rg}', checks=[
            self.check('properties.friendlyName', '{sa2}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('name', '{container_name}'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type {type}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{sa1}'])", 1),
            self.check("length([?properties.friendlyName == '{sa2}'])", 1)])

        self.cmd('backup protection disable -g {rg} -v {vault} -c {sa1} -i {item1} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup protection disable -g {rg} -v {vault} -c {sa2} -i {item2} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup container unregister -g {rg} -v {vault} -c {sa1} --yes --backup-management-type AzureStorage')
        self.cmd('backup container unregister -g {rg} -v {vault} -c {sa2} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @FileSharePreparer(parameter_name="afs2")
    @AFSPolicyPreparer()
    @AFSPolicyPreparer(parameter_name="newpolicy")
    @AFSItemPreparer()
    @AFSItemPreparer(afs_parameter_name="afs2")
    def test_afs_backup_item(self, resource_group, vault_name, storage_account, afs_name, policy_name, afs2, newpolicy):
        self.kwargs.update({
            'vault': vault_name,
            'item1': afs_name,
            'item2': afs2,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'policy': policy_name,
            'newpolicy': newpolicy
        })

        self.kwargs['container1'] = self.cmd('backup container show -g {rg} -v {vault} -n {container} --backup-management-type {type} --query name').get_output_in_json()

        item1_json = self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item1}', checks=[
            self.check('properties.friendlyName', '{item1}'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.kwargs['item1_fullname'] = item1_json['name']

        self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item1_fullname}', checks=[
            self.check('properties.friendlyName', '{item1}'),
            self.check('properties.protectionState', 'IRPending'),
            self.check('properties.protectionStatus', 'Healthy'),
            self.check('resourceGroup', '{rg}')
        ])

        self.cmd('backup item list -g {rg} -v {vault} -c {container}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{item1}'])", 1)
        ])

        self.cmd('backup item list -g {rg} -v {vault} -c {container}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{item2}'])", 1)
        ])

        self.cmd('backup item list -g {rg} -v {vault} -c {container1}', checks=[
            self.check("length(@)", 2),
            self.check("length([?properties.friendlyName == '{item1}'])", 1)
        ])

        self.cmd('backup item set-policy -g {rg} -v {vault} -c {container1} -n {item1} -p {newpolicy}', checks=[
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        item1_json = self.cmd('backup item show -g {rg} -v {vault} -c {container1} -n {item1}').get_output_in_json()
        self.assertIn(self.kwargs['newpolicy'].lower(), item1_json['properties']['policyId'].split('/')[-1].lower())
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item2} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    @AFSItemPreparer()
    @AFSRPPreparer()
    def test_afs_backup_rp(self, resource_group, vault_name, storage_account, afs_name, policy_name, item_name, rp_name):
        self.kwargs.update({
            'vault': vault_name,
            'subscription_id': subscription_id,
            'item1': afs_name,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
        })
        rp_names = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type {type} --query [].name').get_output_in_json()

        self.kwargs['rp1'] = rp_names[0]

        self.cmd('backup recoverypoint show -g {rg} -v {vault} -c {container} -i {item1} -n {rp1} --backup-management-type {type}', checks=[
            self.check("name", '{rp1}'),
            self.check("resourceGroup", '{rg}')
        ])

        rp_list = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type {type}').get_output_in_json()
        rp_count1 = len(rp_list)

        self.kwargs['retain_date'] = (datetime.utcnow() + timedelta(days=30)).strftime('%d-%m-%Y')
        self.kwargs['job'] = self.cmd('backup protection backup-now -g {rg} -v {vault} -c {container} -i {item1} --retain-until {retain_date} --backup-management-type AzureStorage --query name').get_output_in_json()
        self.cmd('backup job wait -g {rg} -v {vault} -n {job}')

        rp_list = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type {type}').get_output_in_json()
        rp_count2 = len(rp_list)

        self.assertTrue(rp_count1 + 1 == rp_count2)
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FilePreparer()
    @FilePreparer()
    @FilePreparer(parameter_name="file2")
    @FileSharePreparer(file_upload=True, file_parameter_name=['file_name', 'file2'])
    @AFSPolicyPreparer()
    @AFSItemPreparer()
    @AFSRPPreparer()
    @FileSharePreparer(parameter_name="afs2")
    def test_afs_backup_restore(self, resource_group, vault_name, storage_account, file_name, afs_name, afs2, file2):
        self.kwargs.update({
            'vault': vault_name,
            'item1': afs_name,
            'item2': afs2,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'file': file_name,
            'file2': file2
        })

        # full share restore original location

        rp_names = self.cmd('backup recoverypoint list -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type {type} --query [].name').get_output_in_json()
        self.kwargs['rp1'] = rp_names[0]

        trigger_restore_job1_json = self.cmd('backup restore restore-azurefileshare -g {rg} -v {vault} -c {container} -i {item1} -r {rp1} --resolve-conflict Overwrite --restore-mode OriginalLocation', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job1'] = trigger_restore_job1_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job1}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job1}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        # full share alternate location

        trigger_restore_job2_json = self.cmd('backup restore restore-azurefileshare -g {rg} -v {vault} -c {container} -i {item1} -r {rp1} --resolve-conflict Overwrite --restore-mode AlternateLocation --target-storage-account {container} --target-file-share {item2} --target-folder folder1', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job2'] = trigger_restore_job2_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job2}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job2}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        # item level recovery alternate location

        trigger_restore_job3_json = self.cmd('backup restore restore-azurefiles -g {rg} -v {vault} -c {container} -i {item1} -r {rp1} --resolve-conflict Overwrite --restore-mode AlternateLocation --source-file-type File --source-file-path {file} --target-storage-account {container} --target-file-share {item2} --target-folder folder1', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job3'] = trigger_restore_job3_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job3}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job3}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        # item level recovery original location

        trigger_restore_job4_json = self.cmd('backup restore restore-azurefiles -g {rg} -v {vault} -c {container} -i {item1} -r {rp1} --resolve-conflict Overwrite --restore-mode OriginalLocation --source-file-type File --source-file-path {file}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.kwargs['job4'] = trigger_restore_job4_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job4}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job4}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        # item level multiple file restore to original location
        trigger_restore_job5_json = self.cmd('backup restore restore-azurefiles -g {rg} -v {vault} -c {container} -i {item1} -r {rp1} --resolve-conflict Overwrite --restore-mode AlternateLocation --source-file-type File --source-file-path {file} {file2} --target-storage-account {container} --target-file-share {item2} --target-folder folder1', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "InProgress"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()
        self.kwargs['job5'] = trigger_restore_job5_json['name']
        self.cmd('backup job wait -g {rg} -v {vault} -n {job5}')

        self.cmd('backup job show -g {rg} -v {vault} -n {job5}', checks=[
            self.check("properties.entityFriendlyName", '{item1}'),
            self.check("properties.operation", "Restore"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item1} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    def test_afs_backup_protection(self, resource_group, vault_name, storage_account, afs_name, policy_name):
        self.kwargs.update({
            'vault': vault_name,
            'item': afs_name,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'policy': policy_name
        })

        self.cmd('backup protection enable-for-azurefileshare -g {rg} -v {vault} --storage-account {container} --azure-file-share {item} -p {policy}', checks=[
            self.check("properties.entityFriendlyName", '{item}'),
            self.check("properties.operation", "ConfigureBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        item_json = self.cmd('backup item list -g {rg} -v {vault} -c {container} --backup-management-type {type}').get_output_in_json()
        protected_item_count1 = len(item_json)

        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item} --backup-management-type {type} --yes', checks=[
            self.check("properties.entityFriendlyName", '{item}'),
            self.check("properties.operation", "DisableBackup"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup item show -g {rg} -v {vault} -c {container} -n {item} --backup-management-type {type}', checks=[
            self.check("properties.friendlyName", '{item}'),
            self.check("properties.protectionState", "ProtectionStopped"),
            self.check("resourceGroup", '{rg}')
        ])

        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {item} --backup-management-type {type} --delete-backup-data true --yes', checks=[
            self.check("properties.entityFriendlyName", '{item}'),
            self.check("properties.operation", "DeleteBackupData"),
            self.check("properties.status", "Completed"),
            self.check("resourceGroup", '{rg}')
        ])

        item_json = self.cmd('backup item list -g {rg} -v {vault} -c {container} --backup-management-type {type}').get_output_in_json()
        protected_item_count2 = len(item_json)

        self.assertTrue(protected_item_count1 == protected_item_count2 + 1)
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    @AFSItemPreparer()
    def test_afs_backup_policy(self, resource_group, vault_name, storage_account, afs_name, policy_name, item_name):
        self.kwargs.update({
            'vault': vault_name,
            'item': item_name,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'policy': policy_name,
            'policy2': "clitestafspolicy"
        })

        policies_json = self.cmd('backup policy list -g {rg} -v {vault} --backup-management-type {type}').get_output_in_json()
        policy_count1 = len(policies_json)

        self.kwargs['policy1_json'] = self.cmd('backup policy show -g {rg} -v {vault} -n {policy}', checks=[
            self.check("name", '{policy}'),
            self.check("properties.backupManagementType", '{type}'),
            self.check("resourceGroup", '{rg}')
        ]).get_output_in_json()

        self.kwargs['policy1_json']['name'] = self.kwargs['policy2']
        self.kwargs['policy1_json']['properties']['retentionPolicy']['dailySchedule']['retentionDuration']['count'] = 25
        self.kwargs['policy1_json'] = json.dumps(self.kwargs['policy1_json'])

        self.cmd("backup policy create -g {rg} -v {vault} --policy '{policy1_json}' --name {policy2} --backup-management-type {type}")

        self.kwargs['policy2_json'] = self.cmd('backup policy show -g {rg} -v {vault} --n {policy2}', checks=[
            self.check("name", '{policy2}'),
            self.check("properties.backupManagementType", '{type}'),
            self.check("properties.protectedItemsCount", 0),
            self.check("properties.retentionPolicy.dailySchedule.retentionDuration.count", 25),
        ]).get_output_in_json()

        policies_json = self.cmd('backup policy list -g {rg} -v {vault} --backup-management-type {type}').get_output_in_json()
        policy_count2 = len(policies_json)
        self.assertTrue(policy_count2 == policy_count1 + 1)

        self.kwargs['policy2_json']['properties']['retentionPolicy']['dailySchedule']['retentionDuration']['count'] = 20
        self.kwargs['policy2_json'] = json.dumps(self.kwargs['policy2_json'])

        self.cmd("backup policy set -g {rg} -v {vault} --policy '{policy2_json}' -n {policy2}")
        self.cmd('backup policy show -g {rg} -v {vault} -n {policy2}', checks=[
            self.check("properties.retentionPolicy.dailySchedule.retentionDuration.count", 20),
        ]).get_output_in_json()

        self.kwargs['afsitem'] = self.cmd('backup item list -g {rg} -v {vault} -c {container} --backup-management-type AzureStorage --query [0].name').get_output_in_json()
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container} -i {afsitem} --backup-management-type AzureStorage --delete-backup-data true --yes').get_output_in_json()
        self.cmd('backup container unregister -g {rg} -v {vault} -c {container} --yes --backup-management-type AzureStorage')

    @record_only()
    @ResourceGroupPreparer(location="southeastasia", random_name_length=20)
    @VaultPreparer()
    @StorageAccountPreparer(location="southeastasia")
    @FileSharePreparer()
    @AFSPolicyPreparer()
    @AFSItemPreparer()
    def test_afs_unregister_container(self, resource_group, vault_name, storage_account, afs_name, policy_name, item_name):
        self.kwargs.update({
            'vault': vault_name,
            'item': item_name,
            'container': storage_account,
            'rg': resource_group,
            'type': "AzureStorage",
            'policy': policy_name,
            'afs': afs_name
        })

        container_json = self.cmd('backup container show -n {container} -v {vault} -g {rg} --backup-management-type {type}', checks=[
            self.check('properties.friendlyName', '{container}'),
            self.check('properties.healthStatus', 'Healthy'),
            self.check('properties.registrationStatus', 'Registered'),
            self.check('resourceGroup', '{rg}')
        ]).get_output_in_json()

        self.kwargs['container_name'] = container_json['name']

        self.kwargs['item'] = self.cmd('backup item list -g {rg} -v {vault} -c {container} --query [0].name').get_output_in_json()
        self.cmd('backup protection disable -g {rg} -v {vault} -c {container_name} -i {item} --backup-management-type AzureStorage --delete-backup-data true --yes')

        self.cmd('backup container unregister -g {rg} -v {vault} -c {container_name} --yes')

        self.cmd('backup container list -v {vault} -g {rg} --backup-management-type {type}', checks=[
            self.check("length([?properties.friendlyName == '{container}'])", 0)])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import json
import mock
import unittest
import datetime
import dateutil
import dateutil.parser
from azure_devtools.scenario_tests import AllowLargeResponse
from azure.cli.testsdk import ScenarioTest, AADGraphUserReplacer, MOCKED_USER_NAME
from knack.util import CLIError


class ServicePrincipalExpressCreateScenarioTest(ScenarioTest):

    @AllowLargeResponse(8192)
    def test_sp_create_scenario(self):
        self.kwargs['display_name'] = self.create_random_name('clisp-test-', 20)
        self.kwargs['app_id_uri'] = 'http://' + self.kwargs['display_name']
        # create app through express option
        self.cmd('ad sp create-for-rbac -n {display_name} --skip-assignment',
                 checks=self.check('name', '{app_id_uri}'))

        # show/list app
        self.cmd('ad app show --id {app_id_uri}',
                 checks=self.check('identifierUris[0]', '{app_id_uri}'))
        self.cmd('ad app list --identifier-uri {app_id_uri}', checks=[
            self.check('[0].identifierUris[0]', '{app_id_uri}'),
            self.check('length([*])', 1)
        ])
        self.assertTrue(len(self.cmd('ad app list').get_output_in_json()) <= 100)
        self.cmd('ad app list --show-mine')
        # show/list sp
        self.cmd('ad sp show --id {app_id_uri}',
                 checks=self.check('servicePrincipalNames[0]', '{app_id_uri}'))
        self.cmd('ad sp list --spn {app_id_uri}', checks=[
            self.check('[0].servicePrincipalNames[0]', '{app_id_uri}'),
            self.check('length([*])', 1),
        ])
        self.cmd('ad sp credential reset -n {app_id_uri}',
                 checks=self.check('name', '{app_id_uri}'))
        # cleanup
        self.cmd('ad sp delete --id {app_id_uri}')  # this would auto-delete the app as well
        self.cmd('ad sp list --spn {app_id_uri}',
                 checks=self.is_empty())
        self.cmd('ad app list --identifier-uri {app_id_uri}',
                 checks=self.is_empty())
        self.assertTrue(len(self.cmd('ad sp list').get_output_in_json()) <= 100)
        self.cmd('ad sp list --show-mine')

    def test_native_app_create_scenario(self):
        self.kwargs = {
            'native_app_name': self.create_random_name('cli-native-', 20),
            'required_access': json.dumps([
                {
                    "resourceAppId": "00000002-0000-0000-c000-000000000000",
                    "resourceAccess": [
                        {
                            "id": "5778995a-e1bf-45b8-affa-663a9f3f4d04",
                            "type": "Scope"
                        }
                    ]
                }]),
            'required_access2': json.dumps([
                {
                    "resourceAppId": "00000002-0000-0000-c000-000000000000",
                    "resourceAccess": [
                        {
                            "id": "311a71cc-e848-46a1-bdf8-97ff7156d8e6",
                            "type": "Scope"
                        }
                    ]
                }
            ])
        }
        result = self.cmd("ad app create --display-name {native_app_name} --native-app --required-resource-accesses '{required_access}'", checks=[
            self.check('publicClient', True),
            self.check('requiredResourceAccess|[0].resourceAccess|[0].id',
                       '5778995a-e1bf-45b8-affa-663a9f3f4d04')
        ]).get_output_in_json()
        self.kwargs['id'] = result['appId']

        # TODO: https://github.com/Azure/azure-cli/pull/13769 fails to work
        # Cert created with
        # openssl req -newkey rsa:2048 -nodes -keyout key.pem -x509 -days 10000 -out certificate.pem
        self.kwargs['cert_file'] = """
MIIDazCCAlOgAwIBAgIUIp5vybhHfKN+ZKL28AntYKhlKXkwDQYJKoZIhvcNAQEL
BQAwRTELMAkGA1UEBhMCQVUxEzARBgNVBAgMClNvbWUtU3RhdGUxITAfBgNVBAoM
GEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZDAeFw0yMDA3MjIwNzE3NDdaFw00NzEy
MDgwNzE3NDdaMEUxCzAJBgNVBAYTAkFVMRMwEQYDVQQIDApTb21lLVN0YXRlMSEw
HwYDVQQKDBhJbnRlcm5ldCBXaWRnaXRzIFB0eSBMdGQwggEiMA0GCSqGSIb3DQEB
AQUAA4IBDwAwggEKAoIBAQDMa00H+/p4RP4Eo//1J81Wowo4y1SKOJHbJ6T/lZ73
5FzFX52gdQ/7HalJOwQdbha78RPGA7bXxEmyEo+q3w+IMYzrqboX5S9yf0v1DZvj
a/VEMtUsq79d7NUUEd+smkuqDxDHFIkMeMM8cXy6tc+TPbc28BkQQiKbzOEZDwy4
HPd7FCqCwwcZtgxfxFQx5A2DkAXtT53zQD8k1zY4UQWhkKDcgvINzQfYxJmUbXqH
27MuJuejhpWLjmwEFCQtMJMrEv44YmlDzmL64iN5HFckO65ikV9fe9g9EcR5acSY
2bsO8WyFYzTffVXFpFF011Vi4d/U0h4wSwj5KLMYMHkfAgMBAAGjUzBRMB0GA1Ud
DgQWBBQxgpSKG7fwIHEopaRA10GB8Z8SOTAfBgNVHSMEGDAWgBQxgpSKG7fwIHEo
paRA10GB8Z8SOTAPBgNVHRMBAf8EBTADAQH/MA0GCSqGSIb3DQEBCwUAA4IBAQAt
I5vbHGxVV3qRtd9PEFe9dUb9Yv9YIa5RUd5l795cgr6qyELfg3xTPZbNf1oUHpGX
NCfm1uqNTorIKOIEoTpA+STVwST/xcqzB6VjS31I/5IIrdK2NQenM+0DVJa+yGhX
+zI3+X3cO2YbyLSKBYqdMsqgnMS/ZC0NnrvigHgq2SC4Vzg8yz5rorjvLJ6ndeht
oWOtdCJKUTPihNh4e+GM2A7UNKdt5WKCiS/n/lShvm+8JEG2lXQmmxR6DOjdDyC4
/6tf7Ln7YoZZ0q6ICp04oMF6bvgGosdOkQATW4X97EmcfIBfHPX2w/Xn47np2rZr
lBMWCjI8gO6W8YQMu7AH""".replace('\n', '')

        result = self.cmd("ad app create --display-name {native_app_name} --key-value {cert_file}", checks=[
            self.check('length(keyCredentials)', 1)
        ])

        try:
            self.cmd("ad app update --id {id} --required-resource-accesses '{required_access2}'")
            self.cmd('ad app show --id {id}', checks=[
                self.check('publicClient', True),
                self.check('requiredResourceAccess|[0].resourceAccess|[0].id',
                           '311a71cc-e848-46a1-bdf8-97ff7156d8e6')
            ])
        except Exception:
            self.cmd('ad app delete --id {id}')

    def test_web_app_no_identifier_uris_other_tenants_create_scenario(self):
        self.kwargs = {
            'web_app_name': self.create_random_name('cli-web-', 20)
        }

        self.cmd("ad app create --display-name {web_app_name} --available-to-other-tenants true", checks=[
            self.exists('appId')
        ])

    def test_app_create_idempotent(self):
        self.kwargs = {
            'display_name': self.create_random_name('app', 20)
        }
        app_id = None
        try:
            result = self.cmd("ad app create --display-name {display_name} --available-to-other-tenants true").get_output_in_json()
            app_id = result['appId']
            self.cmd("ad app create --display-name {display_name} --available-to-other-tenants false",
                     checks=self.check('availableToOtherTenants', False))
        finally:
            self.cmd("ad app delete --id " + app_id)

    def test_sp_show_exit_code(self):
        with self.assertRaises(SystemExit):
            self.assertEqual(self.cmd('ad sp show --id non-exist-sp-name').exit_code, 3)
            self.assertEqual(self.cmd('ad sp show --id 00000000-0000-0000-0000-000000000000').exit_code, 3)


class ApplicationSetScenarioTest(ScenarioTest):

    def test_application_set_scenario(self):
        name = self.create_random_name(prefix='cli-graph', length=14)
        self.kwargs.update({
            'app': 'http://' + name,
            'name': name,
            'app_roles': json.dumps([
                {
                    "allowedMemberTypes": [
                        "User"
                    ],
                    "description": "Approvers can mark documents as approved",
                    "displayName": "Approver",
                    "isEnabled": "true",
                    "value": "approver"
                }
            ])
        })
        app_id = None
        # create app through general option
        self.cmd('ad app create --display-name {name} --homepage {app} --identifier-uris {app}',
                 checks=self.check('identifierUris[0]', '{app}'))

        # set app password
        result = self.cmd('ad app credential reset --id {app} --append --password "test" --years 2').get_output_in_json()
        app_id = result['appId']
        self.assertTrue(app_id)
        self.kwargs['app_id'] = app_id

        try:
            # show by identifierUri
            self.cmd('ad app show --id {app}', checks=self.check('identifierUris[0]', '{app}'))
            # show by appId
            self.cmd('ad app show --id {app_id}', checks=self.check('appId', '{app_id}'))

            self.cmd('ad app list --display-name {name}', checks=[
                self.check('[0].identifierUris[0]', '{app}'),
                self.check('length([*])', 1)
            ])

            # update app
            self.kwargs['reply_uri'] = "http://azureclitest-replyuri"
            self.kwargs['reply_uri2'] = "http://azureclitest-replyuri2"
            self.cmd('ad app update --id {app} --reply-urls {reply_uri}')
            self.cmd('ad app show --id {app}',
                     checks=self.check('replyUrls[0]', '{reply_uri}'))

            # add and remove replyUrl
            self.cmd('ad app update --id {app} --add replyUrls {reply_uri2}')
            self.cmd('ad app show --id {app}', checks=self.check('length(replyUrls)', 2))
            self.cmd('ad app update --id {app} --remove replyUrls 1')
            self.cmd('ad app show --id {app}', checks=[
                self.check('length(replyUrls)', 1),
                self.check('replyUrls[0]', '{reply_uri2}')
            ])

            # update displayName
            name2 = self.create_random_name(prefix='cli-graph', length=14)
            self.kwargs['name2'] = name2
            self.cmd('ad app update --id {app} --display-name {name2}')
            self.cmd('ad app show --id {app}', checks=self.check('displayName', '{name2}'))

            # update homepage
            self.kwargs['homepage2'] = 'http://' + name2
            self.cmd('ad app update --id {app} --homepage {homepage2}')
            self.cmd('ad app show --id {app}', checks=self.check('homepage', '{homepage2}'))

            # invoke generic update
            self.cmd('ad app update --id {app} --set oauth2AllowUrlPathMatching=true')
            self.cmd('ad app show --id {app}',
                     checks=self.check('oauth2AllowUrlPathMatching', True))

            # update app_roles
            self.cmd("ad app update --id {app} --app-roles '{app_roles}'")
            self.cmd('ad app show --id {app}',
                     checks=self.check('length(appRoles)', 1))

            # delete app
            self.cmd('ad app delete --id {app}')
            app_id = None
            self.cmd('ad app list --identifier-uri {app}',
                     checks=self.is_empty())
        finally:
            if app_id:
                self.cmd("ad app delete --id " + app_id)

    def test_app_show_exit_code(self):
        with self.assertRaises(SystemExit):
            self.assertEqual(self.cmd('ad app show --id non-exist-identifierUris').exit_code, 3)
            self.assertEqual(self.cmd('ad app show --id 00000000-0000-0000-0000-000000000000').exit_code, 3)

    def test_application_optional_claims(self):
        name1 = self.create_random_name(prefix='cli-app-', length=14)
        name2 = self.create_random_name(prefix='cli-app-', length=14)
        self.kwargs.update({
            'name1': name1,
            'name2': name2,
            'optional_claims': json.dumps({
                "idToken": [
                    {
                        "name": "auth_time",
                        "source": None,
                        "essential": False
                    }
                ],
                "accessToken": [
                    {
                        "name": "email",
                        "source": None,
                        "essential": False
                    }
                ]
            })
        })
        self.cmd("ad app create --display-name {name1} --optional-claims '{optional_claims}'",
                 checks=[
                     self.check('displayName', '{name1}'),
                     self.check('length(optionalClaims.idToken)', 1),
                     self.check('length(optionalClaims.accessToken)', 1)
                 ])
        app_id = self.cmd('ad app create --display-name {name2}').get_output_in_json()['appId']
        self.kwargs.update({
            'app_id': app_id
        })
        self.cmd("ad app update --id {app_id} --optional-claims '{optional_claims}'")
        self.cmd('ad app show --id {app_id}',
                 checks=[
                     self.check('displayName', '{name2}'),
                     self.check('length(optionalClaims.idToken)', 1),
                     self.check('length(optionalClaims.accessToken)', 1)
                 ])


class CreateForRbacScenarioTest(ScenarioTest):

    @AllowLargeResponse()
    def test_revoke_sp_for_rbac(self):
        self.kwargs['display_name'] = self.create_random_name(prefix='cli-graph', length=14)
        self.kwargs['name'] = 'http://' + self.kwargs['display_name']

        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            self.cmd('ad sp create-for-rbac -n {display_name}')

            self.cmd('ad sp list --spn {name}')

            self.cmd('ad app list --identifier-uri {name}')

            result = self.cmd('role assignment list --assignee {name}').get_output_in_json()
            object_id = result[0]['principalId']

            self.cmd('ad sp delete --id {name}')

            result2 = self.cmd('role assignment list --all').get_output_in_json()
            self.assertFalse([a for a in result2 if a['id'] == object_id])

            self.cmd('ad sp list --spn {name}',
                     checks=self.check('length([])', 0))
            self.cmd('ad app list --identifier-uri {name}',
                     checks=self.check('length([])', 0))

    @AllowLargeResponse()
    def test_create_for_rbac_idempotent(self):
        self.kwargs['display_name'] = self.create_random_name(prefix='sp_', length=14)
        self.kwargs['name'] = 'http://' + self.kwargs['display_name']

        with mock.patch('azure.cli.command_modules.role.custom._gen_guid', side_effect=self.create_guid):
            try:
                self.cmd('ad sp create-for-rbac -n {display_name}')
                self.cmd('ad sp create-for-rbac -n {display_name}')
                result = self.cmd('ad sp list --spn {name}').get_output_in_json()
                self.assertEqual(1, len(result))
                result = self.cmd('role assignment list --assignee {name}').get_output_in_json()
                self.assertEqual(1, len(result))
            finally:
                self.cmd('ad sp delete --id {name}')


class GraphGroupScenarioTest(ScenarioTest):

    def test_graph_group_scenario(self):
        username = get_signed_in_user(self)
        if not username:
            return  # this test delete users which are beyond a SP's capacity, so quit...

        domain = username.split('@', 1)[1]
        self.kwargs = {
            'user1': self.create_random_name(prefix='deleteme1', length=15),
            'user2': self.create_random_name(prefix='deleteme2', length=15),
            'domain': domain,
            'new_mail_nick_name': 'deleteme11',
            'group': 'deleteme_g',
            'pass': 'Test1234!!'
        }
        self.recording_processors.append(AADGraphUserReplacer('@' + domain, '@example.com'))
        try:
            # create user1
            user1_result = self.cmd('ad user create --display-name {user1} --password {pass} --user-principal-name {user1}@{domain}').get_output_in_json()
            self.kwargs['user1_id'] = user1_result['objectId']

            # update user1
            self.cmd('ad user update --display-name {user1}_new --account-enabled false --id {user1}@{domain} --mail-nickname {new_mail_nick_name}')
            user1_update_result = self.cmd('ad user show --upn-or-object-id {user1}@{domain}', checks=[self.check("displayName", '{user1}_new'),
                                                                                                       self.check("accountEnabled", False)]).get_output_in_json()
            self.cmd('ad user update --id {user1}@{domain} --password {pass}')
            self.cmd('ad user update --id {user1}@{domain} --password {pass} --force-change-password-next-login true')
            with self.assertRaises(CLIError):
                self.cmd('ad user update --id {user1}@{domain} --force-change-password-next-login false')
            self.kwargs['user1_id'] = user1_update_result['objectId']

            # create user2
            user2_result = self.cmd('ad user create --display-name {user2} --password {pass} --user-principal-name {user2}@{domain}').get_output_in_json()
            self.kwargs['user2_id'] = user2_result['objectId']
            # create group
            group_result = self.cmd('ad group create --display-name {group} --mail-nickname {group} --description {group}').get_output_in_json()
            self.kwargs['group_id'] = group_result['objectId']
            # add user1 into group
            self.cmd('ad group member add -g {group} --member-id {user1_id}',
                     checks=self.is_empty())
            # add user2 into group
            self.cmd('ad group member add -g {group} --member-id {user2_id}',
                     checks=self.is_empty())

            # show user's group memberships
            self.cmd('ad user get-member-groups --upn-or-object-id {user1_id}',
                     checks=self.check('[0].displayName', self.kwargs['group']))
            # show group
            self.cmd('ad group show -g {group}', checks=[
                self.check('objectId', '{group_id}'),
                self.check('displayName', '{group}'),
                self.check('description', '{group}')
            ])
            self.cmd('ad group show -g {group}',
                     checks=self.check('displayName', '{group}'))
            # list group
            self.cmd('ad group list --display-name {group}',
                     checks=self.check('[0].displayName', '{group}'))
            # show member groups
            self.cmd('ad group get-member-groups -g {group}',
                     checks=self.check('length([])', 0))
            # check user1 memebership
            self.cmd('ad group member check -g {group} --member-id {user1_id}',
                     checks=self.check('value', True))

            # check user2 memebership
            self.cmd('ad group member check -g {group} --member-id {user2_id}',
                     checks=self.check('value', True))

            self.cmd('ad group member list -g {group}', checks=[
                self.check("length([?displayName=='{user1}_new'])", 1),
                self.check("length([?displayName=='{user2}'])", 1),
                self.check("length([?displayName=='{user1}'])", 0),
                self.check("length([])", 2),
            ])
            # remove user1
            self.cmd('ad group member remove -g {group} --member-id {user1_id}')
            # check user1 memebership
            self.cmd('ad group member check -g {group} --member-id {user1_id}',
                     checks=self.check('value', False))
            # delete the group
            self.cmd('ad group delete -g {group}')
            self.cmd('ad group list',
                     checks=self.check("length([?displayName=='{group}'])", 0))
        finally:
            try:
                self.cmd('ad user delete --upn-or-object-id {user1_id}')
                self.cmd('ad user delete --upn-or-object-id {user2_id}')
                self.cmd('ad group delete -g {group}')
            except Exception:
                pass

    def test_graph_group_idempotent(self):
        account_info = self.cmd('account show').get_output_in_json()
        if account_info['user']['type'] == 'servicePrincipal':
            return  # this test delete users which are beyond a SP's capacity, so quit...

        self.kwargs = {
            'group': 'deleteme_g2',
        }
        try:
            self.cmd('ad group create --display-name {group} --mail-nickname {group}')
            self.cmd('ad group create --display-name {group} --mail-nickname {group}')
            self.cmd('ad group list',
                     checks=self.check("length([?displayName=='{group}'])", 1))
        finally:
            try:
                self.cmd('ad group delete -g {group}')
            except:
                pass

    def test_graph_group_show(self):
        account_info = self.cmd('account show').get_output_in_json()
        if account_info['user']['type'] == 'servicePrincipal':
            return  # this test delete users which are beyond a SP's capacity, so quit...

        self.kwargs = {
            'group1': 'show_group_1',
            'group11': 'show_group_11',
            'prefix': 'show_prefix',
            'prefix_group': 'show_prefix_group'
        }

        self.cmd('ad group create --display-name {group1} --mail-nickname {group1}')
        self.cmd('ad group create --display-name {group11} --mail-nickname {group11}')
        self.cmd('ad group create --display-name {prefix_group} --mail-nickname {prefix_group}')
        self.cmd('ad group show --group {group1}',
                 checks=self.check('displayName', '{group1}'))
        self.cmd('ad group show --group {group11}',
                 checks=self.check('displayName', '{group11}'))
        self.cmd('ad group show --group {prefix}',
                 checks=self.check('displayName', '{prefix_group}'))
        self.cmd('ad group delete -g {group1}')
        self.cmd('ad group delete -g {group11}')
        self.cmd('ad group delete -g {prefix}')


def get_signed_in_user(test_case):
    playback = not (test_case.is_live or test_case.in_recording)
    if playback:
        return MOCKED_USER_NAME
    else:
        account_info = test_case.cmd('account show').get_output_in_json()
        if account_info['user']['type'] != 'servicePrincipal':
            return account_info['user']['name']
    return None


class GraphOwnerScenarioTest(ScenarioTest):

    def test_graph_application_ownership(self):
        owner = get_signed_in_user(self)
        if not owner:
            return  # this test delete users which are beyond a SP's capacity, so quit...

        self.kwargs = {
            'owner': owner,
            'display_name': self.create_random_name('sp', 15)
        }
        self.recording_processors.append(AADGraphUserReplacer(owner, 'example@example.com'))
        try:
            self.kwargs['owner_object_id'] = self.cmd('ad user show --upn-or-object-id {owner}').get_output_in_json()['objectId']
            self.kwargs['app_id'] = self.cmd('ad sp create-for-rbac -n {display_name} --skip-assignment').get_output_in_json()['appId']
            self.cmd('ad app owner add --owner-object-id {owner_object_id} --id {app_id}')
            self.cmd('ad app owner add --owner-object-id {owner_object_id} --id {app_id}')  # test idempotent
            self.cmd('ad app owner list --id {app_id}', checks=self.check('[0].userPrincipalName', owner))
            self.cmd('ad app owner remove --owner-object-id {owner_object_id} --id {app_id}')
            self.cmd('ad app owner list --id {app_id}', checks=self.check('length([*])', 0))
        finally:
            if self.kwargs['app_id']:
                self.cmd('ad sp delete --id {app_id}')

    def test_graph_group_ownership(self):
        owner = get_signed_in_user(self)
        if not owner:
            return  # this test delete users which are beyond a SP's capacity, so quit...

        self.kwargs = {
            'owner': owner,
            'group': self.create_random_name('cli-grp', 15),
        }
        self.recording_processors.append(AADGraphUserReplacer(owner, 'example@example.com'))
        try:
            self.kwargs['owner_object_id'] = self.cmd('ad user show --upn-or-object-id {owner}').get_output_in_json()['objectId']
            self.kwargs['group_object_id'] = self.cmd('ad group create --display-name {group} --mail-nickname {group}').get_output_in_json()['objectId']
            self.cmd('ad group owner add -g {group_object_id} --owner-object-id {owner_object_id}')
            self.cmd('ad group owner add -g {group_object_id} --owner-object-id {owner_object_id}')
            self.cmd('ad group owner list -g {group_object_id}', checks=self.check('length([*])', 1))
        finally:
            if self.kwargs['group_object_id']:
                self.cmd('ad group delete -g ' + self.kwargs['group_object_id'])


class GraphAppCredsScenarioTest(ScenarioTest):
    def test_graph_app_cred_e2e(self):
        if not get_signed_in_user(self):
            return  # this test delete users which are beyond a SP's capacity, so quit...

        self.kwargs = {
            'display_name': self.create_random_name('cli-app-', 15),
            'display_name2': self.create_random_name('cli-app-', 15),
            'test_pwd': 'verysecretpwd123*'
        }
        self.kwargs['app'] = 'http://' + self.kwargs['display_name']
        self.kwargs['app2'] = 'http://' + self.kwargs['display_name2']
        app_id = None
        app_id2 = None
        try:
            result = self.cmd('ad sp create-for-rbac --name {display_name} --skip-assignment').get_output_in_json()
            app_id = result['appId']

            result = self.cmd('ad sp credential list --id {app}').get_output_in_json()
            key_id = result[0]['keyId']
            self.cmd('ad sp credential reset -n {app} --password {test_pwd} --append --credential-description newCred1')
            self.cmd('ad sp credential list --id {app}', checks=[
                self.check('length([*])', 2),
                self.check('[0].customKeyIdentifier', 'newCred1'),
                self.check('[1].customKeyIdentifier', 'rbac')  # auto configured by create-for-rbac
            ])
            self.cmd('ad sp credential delete --id {app} --key-id ' + key_id)
            result = self.cmd('ad sp credential list --id {app}', checks=self.check('length([*])', 1)).get_output_in_json()
            self.assertTrue(result[0]['keyId'] != key_id)

            # try the same through app commands
            result = self.cmd('ad app credential list --id {app}', checks=self.check('length([*])', 1)).get_output_in_json()
            key_id = result[0]['keyId']
            self.cmd('ad app credential reset --id {app} --password {test_pwd} --append --credential-description newCred2')
            result = self.cmd('ad app credential list --id {app}', checks=[
                self.check('length([*])', 2),
                self.check('[0].customKeyIdentifier', 'newCred2'),
                self.check('[1].customKeyIdentifier', 'newCred1')
            ])
            self.cmd('ad app credential delete --id {app} --key-id ' + key_id)
            self.cmd('ad app credential list --id {app}', checks=self.check('length([*])', 1))

            # try use --end-date
            self.cmd('ad sp credential reset -n {app} --password {test_pwd} --end-date "2100-12-31T11:59:59+00:00" --credential-description newCred3')
            self.cmd('ad app credential reset --id {app} --password {test_pwd} --end-date "2100-12-31" --credential-description newCred4')

            # ensure we can update other properties #7728
            self.cmd('ad app update --id {app} --set groupMembershipClaims=All')
            self.cmd('ad app show --id {app}', checks=self.check('groupMembershipClaims', 'All'))

            # ensure we can update SP's properties #5948
            self.cmd('az ad sp update --id {app} --set appRoleAssignmentRequired=true')
            self.cmd('az ad sp show --id {app}')

            result = self.cmd('ad sp create-for-rbac --name {display_name2} --skip-assignment --years 10').get_output_in_json()
            app_id2 = result['appId']
            result = self.cmd('ad sp credential list --id {app2}', checks=self.check('length([*])', 1)).get_output_in_json()
            diff = dateutil.parser.parse(result[0]['endDate']).replace(tzinfo=None) - datetime.datetime.utcnow()
            self.assertTrue(diff.days > 1)  # it is just a smoke test to verify the credential does get applied
        finally:
            if app_id:
                self.cmd('ad app delete --id ' + app_id)
            if app_id2:
                self.cmd('ad app delete --id ' + app_id2)


class GraphAppRequiredAccessScenarioTest(ScenarioTest):

    def test_graph_required_access_e2e(self):
        if not get_signed_in_user(self):
            return  # this test delete users which are beyond a SP's capacity, so quit...
        self.kwargs = {
            'display_name': self.create_random_name('cli-app-', 15),
            'graph_resource': '00000002-0000-0000-c000-000000000000',
            'target_api': 'a42657d6-7f20-40e3-b6f0-cee03008a62a',
            'target_api2': '311a71cc-e848-46a1-bdf8-97ff7156d8e6'
        }
        app_id = None
        try:
            result = self.cmd('ad sp create-for-rbac --name {display_name} --skip-assignment').get_output_in_json()
            self.kwargs['app_id'] = result['appId']
            app_id = result['appId']
            self.cmd('ad app permission add --id {app_id} --api {graph_resource} --api-permissions {target_api}=Scope')
            self.cmd('ad app permission grant --id {app_id} --api {graph_resource}')
            permissions = self.cmd('ad app permission list --id {app_id}', checks=[
                self.check('length([*])', 1)
            ]).get_output_in_json()
            self.assertTrue(dateutil.parser.parse(permissions[0]['expiryTime']))  # verify it is a time
            self.cmd('ad app permission list-grants --id {app_id}', checks=self.check('length([*])', 1))
            self.cmd('ad app permission list-grants --id {app_id} --show-resource-name',
                     checks=self.check('[0].resourceDisplayName', "Windows Azure Active Directory"))
            self.cmd('ad app permission add --id {app_id} --api {graph_resource} --api-permissions {target_api2}=Scope')
            self.cmd('ad app permission grant --id {app_id} --api {graph_resource}')
            self.cmd('ad app permission delete --id {app_id} --api {graph_resource}')
            self.cmd('ad app permission list --id {app_id}', checks=self.check('length([*])', 0))
        finally:
            if app_id:
                try:
                    self.cmd('ad app delete --id ' + app_id)
                except:
                    pass

    @AllowLargeResponse()
    def test_graph_permission(self):
        if not get_signed_in_user(self):
            return
        self.kwargs = {
            'display_name': self.create_random_name('cli-app-', 15),
            # AD Graph
            'ad_graph_resource': '00000002-0000-0000-c000-000000000000',
            # Delegated Directory.AccessAsUser.All
            'ad_target_api': 'a42657d6-7f20-40e3-b6f0-cee03008a62a',
            # Delegated User.Read
            'ad_target_api2': '311a71cc-e848-46a1-bdf8-97ff7156d8e6',
            # MS Graph
            'ms_graph_resource': '00000003-0000-0000-c000-000000000000',
            # Delegated Directory.AccessAsUser.All
            'ms_target_api': '0e263e50-5827-48a4-b97c-d940288653c7',
            # Delegated User.Read
            'ms_target_api2': 'e1fe6dd8-ba31-4d61-89e7-88639da4683d'
        }
        app_id = None
        try:
            result = self.cmd('ad sp create-for-rbac --name {display_name} --skip-assignment').get_output_in_json()
            self.kwargs['app_id'] = result['appId']
            app_id = result['appId']

            # Test add permissions using a list
            self.cmd('ad app permission add --id {app_id} --api {ad_graph_resource} '
                     '--api-permissions {ad_target_api}=Scope {ad_target_api2}=Scope')
            self.cmd('ad app permission add --id {app_id} --api {ms_graph_resource} '
                     '--api-permissions {ms_target_api}=Scope {ms_target_api2}=Scope')
            permissions = self.cmd('ad app permission list --id {app_id}', checks=[self.check('length([*])', 2)]).get_output_in_json()
            # Sample result (required_resource_access):
            # [
            #     {
            #         "additionalProperties": null,
            #         "expiryTime": "",
            #         "resourceAccess": [
            #             {
            #                 "additionalProperties": null,
            #                 "id": "a42657d6-7f20-40e3-b6f0-cee03008a62a",
            #                 "type": "Scope"
            #             },
            #             {
            #                 "additionalProperties": null,
            #                 "id": "311a71cc-e848-46a1-bdf8-97ff7156d8e6",
            #                 "type": "Scope"
            #             }
            #         ],
            #         "resourceAppId": "00000002-0000-0000-c000-000000000000"
            #     },
            #     {
            #         "additionalProperties": null,
            #         "expiryTime": "",
            #         "resourceAccess": [
            #             {
            #                 "additionalProperties": null,
            #                 "id": "0e263e50-5827-48a4-b97c-d940288653c7",
            #                 "type": "Scope"
            #             },
            #             {
            #                 "additionalProperties": null,
            #                 "id": "e1fe6dd8-ba31-4d61-89e7-88639da4683d",
            #                 "type": "Scope"
            #             }
            #         ],
            #         "resourceAppId": "00000003-0000-0000-c000-000000000000"
            #     }
            # ]

            ad_target_api_object = {
                "additionalProperties": None,
                "id": self.kwargs['ad_target_api'],
                "type": "Scope"}
            ad_target_api2_object = {
                "additionalProperties": None,
                "id": self.kwargs['ad_target_api2'],
                "type": "Scope"}
            ms_target_api_object = {
                "additionalProperties": None,
                "id": self.kwargs['ms_target_api'],
                "type": "Scope"}
            ms_target_api2_object = {
                "additionalProperties": None,
                "id": self.kwargs['ms_target_api2'],
                "type": "Scope"}

            def get_required_resource_access(required_resource_access_list, resource_app_id):
                """Search for the RequiredResourceAccess from required_resource_access(list) by resourceAppId."""
                return next(
                    filter(lambda a: a['resourceAppId'] == resource_app_id, required_resource_access_list), None)

            ad_api = get_required_resource_access(permissions, self.kwargs['ad_graph_resource'])
            ms_api = get_required_resource_access(permissions, self.kwargs['ms_graph_resource'])

            # Check initial `permission add` is correct
            self.assertEqual(ad_api['resourceAccess'], [ad_target_api_object, ad_target_api2_object])
            self.assertEqual(ms_api['resourceAccess'], [ms_target_api_object, ms_target_api2_object])

            # Test delete 1 api-permission (ResourceAccess) ms_target_api.
            self.cmd('ad app permission delete --id {app_id} --api {ms_graph_resource} --api-permissions {ms_target_api}')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            ms_api = get_required_resource_access(permissions, self.kwargs['ms_graph_resource'])
            # ms_target_api (ResourceAccess) is deleted and ms_target_api2 (ResourceAccess) remains
            self.assertEqual(ms_api['resourceAccess'], [ms_target_api2_object])

            # Test delete 1 api-permission ms_target_api2 (ResourceAccess)
            self.cmd('ad app permission delete --id {app_id} --api {ms_graph_resource} --api-permissions {ms_target_api2}')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            ms_api = get_required_resource_access(permissions, self.kwargs['ms_graph_resource'])
            # ms_graph_resource (RequiredResourceAccess) is removed automatically
            self.assertIsNone(ms_api)

            # Add ms_target_api and ms_target_api2 back
            self.cmd('ad app permission add --id {app_id} --api {ms_graph_resource} '
                     '--api-permissions {ms_target_api}=Scope {ms_target_api2}=Scope')
            # Delete both ms_target_api and ms_target_api2 at the same time
            self.cmd('ad app permission delete --id {app_id} --api {ms_graph_resource} --api-permissions {ms_target_api} {ms_target_api2}')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            ms_api = get_required_resource_access(permissions, self.kwargs['ms_graph_resource'])
            # ms_graph_resource (RequiredResourceAccess) is removed automatically
            self.assertIsNone(ms_api)

            # Test delete 1 api ad_graph_resource (RequiredResourceAccess)
            self.cmd('ad app permission delete --id {app_id} --api {ad_graph_resource}')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            ad_api = get_required_resource_access(permissions, self.kwargs['ad_graph_resource'])
            self.assertIsNone(ad_api)

            # Test delete non-existing api
            self.cmd('ad app permission delete --id {app_id} --api 11111111-0000-0000-c000-000000000000')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            self.assertEqual(permissions, [])

            # Test delete api permission from non-existing api
            self.cmd('ad app permission delete --id {app_id} --api 11111111-0000-0000-c000-000000000000 --api-permissions {ms_target_api} {ms_target_api2}')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            self.assertEqual(permissions, [])

            # Test delete non-existing api permission from existing api
            self.cmd('ad app permission add --id {app_id} --api {ms_graph_resource} '
                     '--api-permissions {ms_target_api}=Scope {ms_target_api2}=Scope')
            self.cmd('ad app permission delete --id {app_id} --api {ms_graph_resource} --api-permissions 22222222-0000-0000-c000-000000000000')
            permissions = self.cmd('ad app permission list --id {app_id}').get_output_in_json()
            ms_api = get_required_resource_access(permissions, self.kwargs['ms_graph_resource'])
            self.assertEqual(ms_api['resourceAccess'], [ms_target_api_object, ms_target_api2_object])
        finally:
            if app_id:
                try:
                    self.cmd('ad app delete --id ' + app_id)
                except:
                    pass



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import tempfile
import zipfile
import tarfile
import shutil
import unittest

from azure.cli.core.extension import WheelExtension
try:
    from azure.cli.core.extension.tests.latest import ExtensionTypeTestMixin, get_test_data_file
except ImportError:
    from . import ExtensionTypeTestMixin, get_test_data_file


class TestWheelTypeExtensionMetadata(ExtensionTypeTestMixin):

    def test_reading_wheel_type_0_30_0_extension_metadata(self):
        """
        Test wheel==0.30.0 containing metadata.json and we can handle it properly.
        For scenario like 'az extenion add'.
        """

        # this wheel contains metadata.json and METADATA
        wheel_0_30_0_packed = get_test_data_file('wheel_0_30_0_packed_extension-0.1.0-py3-none-any.whl')

        zf = zipfile.ZipFile(wheel_0_30_0_packed)
        zf.extractall(self.ext_dir)

        ext_name, ext_version = 'hello', '0.1.0'

        whl_extension = WheelExtension(ext_name, self.ext_dir)
        metadata = whl_extension.get_metadata()  # able to read metadata from wheel==0.30.0 built extension

        # wheel type extension generates .dist-info
        dist_info = ext_name + '-' + ext_version + '.dist-info'

        # assert Python metadata
        self.assertEqual(metadata['name'], ext_name)
        self.assertEqual(metadata['version'], ext_version)
        self.assertEqual(metadata['author'], 'Microsoft Corporation')
        self.assertIn('metadata.json', os.listdir(os.path.join(self.ext_dir, dist_info)))

        # assert Azure CLI extended metadata
        self.assertTrue(metadata['azext.isPreview'])
        self.assertTrue(metadata['azext.isExperimental'])
        self.assertEqual(metadata['azext.minCliCoreVersion'], '2.0.67')

    def test_reading_wheel_type_0_31_0_extension_metadata(self):
        """
        Test wheel>=0.31.0 not containing metadata.json but we can still handle it properly.
        For scenario like 'az extenion add'.
        """

        # this wheel contains METADATA only
        wheel_0_31_0_packed = get_test_data_file('wheel_0_31_0_packed_extension-0.1.0-py3-none-any.whl')

        zf = zipfile.ZipFile(wheel_0_31_0_packed)
        zf.extractall(self.ext_dir)

        ext_name, ext_version = 'hello', '0.1.0'

        whl_extension = WheelExtension(ext_name, self.ext_dir)
        metadata = whl_extension.get_metadata()  # able to read metadata from wheel==0.30.0 built extension

        # wheel type extension generates .dist-info
        dist_info = ext_name + '-' + ext_version + '.dist-info'

        # assert Python metadata
        self.assertEqual(metadata['name'], ext_name)
        self.assertEqual(metadata['version'], ext_version)
        self.assertEqual(metadata['author'], 'Microsoft Corporation')
        self.assertNotIn('metadata.json', os.listdir(os.path.join(self.ext_dir, dist_info)))

        # assert Azure CLI extended metadata
        self.assertTrue(metadata['azext.isPreview'])
        self.assertTrue(metadata['azext.isExperimental'])
        self.assertEqual(metadata['azext.minCliCoreVersion'], '2.0.67')

    def test_reading_wheel_type_extension_from_develop_mode(self):
        """
        Test wheel type extension but installing from source code.
        For scenario that user are developing extension via 'pip install -e' directlly
        and load it from _CUSTOM_EXT_DIR or GLOBAL_CONFIG_DIR
        """

        source_code_packaged = get_test_data_file('hello-0.1.0.tar.gz')

        with tarfile.open(source_code_packaged, 'r:gz') as tar:
            tar.extractall(self.ext_dir)

        ext_name, ext_version = 'hello', '0.1.0'

        ext_extension = WheelExtension(ext_name, os.path.join(self.ext_dir, ext_name + '-' + ext_version))
        metadata = ext_extension.get_metadata()  # able to read metadata from source code even in wheel type extension

        # assert Python metadata
        self.assertEqual(metadata['name'], ext_name)
        self.assertEqual(metadata['version'], ext_version)
        self.assertEqual(metadata['author'], 'Microsoft Corporation')
        self.assertNotIn('metadata.json', os.listdir(os.path.join(self.ext_dir, ext_name + '-' + ext_version)))

        # assert Azure CLI extended metadata
        self.assertTrue(metadata['azext.isPreview'])
        self.assertTrue(metadata['azext.isExperimental'])
        self.assertEqual(metadata['azext.minCliCoreVersion'], '2.0.67')



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
#
# This script is used to generate history notes for the commits on dev branch since last release.
# The history notes are generated based on the title/description of the Pull Requests for the commits.
# Make sure you have added the remote repository as upstream and fetched the latest code, i.e. you
# have done:
# git remote add upstream git@github.com:Azure/azure-cli.git
# git fetch upstream

import fileinput
import json
import os
import re
import subprocess
import requests
from requests.auth import HTTPBasicAuth

base_url = 'https://api.github.com/repos/azure/azure-cli'
commit_pr_url = '{}/commits/commit_id/pulls'.format(base_url)

history_line_breaker = '==============='
history_notes = {}

# key is lower case and removed spaces of a component
# value is the recommended format of a component
customized_dict = {
    "privatedns": "Private DNS",
    "appconfig": "App Config",
}

# This dict will be filled with all historical compenents.
# key is lower case and removed spaces of a component
# value is the recommended format of a component, when there're multiple formats,
# the one with spaces will be picked, i.e. pick 'Key Vault' over 'KeyVault'.
# If the key also exists in customized_dict, the value will be overwritten with customized_dict[key].
component_dict = {}

def get_component_dict():
    with open('src/azure-cli/HISTORY.rst', 'r', encoding="utf-8") as history_file:
        for line in history_file:
            result = re.search(r'^\*\*(.*)\*\*$', line)
            if result is not None:
                comp = result.group(1)
                key = comp.lower().replace(' ', '')
                if key in customized_dict:
                    component_dict[key] = customized_dict[key]
                elif key in component_dict:
                    if ' ' not in component_dict[key] and ' ' in comp:
                        component_dict[key] = comp
                else:
                    component_dict[key] = comp
    print(component_dict)

def generate_history_notes():
    get_component_dict()
    dev_commits = get_commits()
    print("Get PRs for {} commits.".format(len(dev_commits)))
    for commit in dev_commits:
        prs = get_prs_for_commit(commit['sha'])
        # parse PR if one commit is mapped to one PR
        if len(prs) == 1:
            process_pr(prs[0])
        else:
            process_commit(commit)

    cli_history = ''
    core_history = ''
    for component in sorted(history_notes, key=str.casefold):
        if component.casefold() == 'core':
            core_history += construct_core_history(component)
        else:
            cli_history += construct_cli_history(component)
    if core_history == '':
        core_history = '* Minor fixes\n'
    else:
        core_history = core_history[:-1]  # remove last \n

    print("azure-cli history notes:")
    print(cli_history)
    print("azure-cli-core history notes:")
    print(core_history)

    cli_history = cli_history[:-1]  # remove last \n
    with fileinput.FileInput('src/azure-cli/HISTORY.rst',
                             inplace=True) as file:
        modify_history_file(file, cli_history)

    with fileinput.FileInput('src/azure-cli-core/HISTORY.rst',
                             inplace=True) as file:
        modify_history_file(file, core_history)


def modify_history_file(file: fileinput.FileInput, new_history: str):
    write = True
    for line in file:
        if line == '{}\n'.format(history_line_breaker):
            print(line.replace(
                history_line_breaker,
                '{}\n\n{}'.format(history_line_breaker, new_history)),
                  end='')
            write = False
        else:
            # remove any history notes written above previous release version
            # make the generation of history notes idempotent
            if re.match(r'^[0-9]+\.[0-9]+\.[0-9]+$', line):
                write = True
            if write:
                print(line, end='')


def construct_cli_history(component: str):
    history = '**{}**\n\n'.format(component)
    for note in history_notes[component]:
        history += '* {}\n'.format(note)
    history += '\n'
    return history


def construct_core_history(component: str):
    history = ''
    for note in history_notes[component]:
        history += '* {}\n'.format(note)
    history += '\n'
    return history


def get_commits():
    last_release = 'azure-cli-{}'.format(os.getenv('PRE_VERSION')) if os.getenv('PRE_VERSION') else 'upstream/release'
    out = subprocess.Popen([
        'git', 'log', '{}..upstream/dev'.format(last_release),
        '--pretty=format:"%H %s"'
    ],
                           stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT)
    stdout, _ = out.communicate()
    dev_commits = []
    for line in stdout.decode('utf-8').splitlines():
        words = line.strip('"').split(None, 1)
        sha = words[0]
        subject = words[1]
        dev_commits.append({'sha': sha, 'subject': subject})
    dev_commits.reverse()
    return dev_commits


def get_prs_for_commit(commit: str):
    headers = {'Accept': 'application/vnd.github.groot-preview+json'}
    url = commit_pr_url.replace('commit_id', commit)
    response = requests.get(url, headers=headers, auth=HTTPBasicAuth('anything', os.environ.get('PAT_TOKEN')))
    if response.status_code != 200:
        raise Exception("Request to {} failed with {}".format(
            url, response.status_code))
    prs = json.loads(response.content.decode('utf-8'))
    return prs


def process_pr(pr):
    lines = [pr['title']]
    body = pr['body']
    content = ''
    search_result = re.search(r'\*\*History Notes\*\*(.*)---',
                              body,
                              flags=re.DOTALL)
    if search_result is None:
        search_result = re.search(r'\*\*History Notes\*\*(.*)',
                                  body,
                                  flags=re.DOTALL)
        if search_result is not None:
            content = search_result.group(1)
    else:
        content = search_result.group(1)
    if content:
        lines.extend(content.splitlines())
    process_lines(lines, pr['number'])


def process_commit(commit):
    lines = commit['subject'].splitlines()
    process_lines(lines)


def process_lines(lines: [str], pr_num: str = None):
    # do not put note of hotfix here since it's for last release
    if re.search('hotfix', lines[0], re.IGNORECASE):
        return
    note_in_desc = False
    for desc in lines[1:]:
        component, note = parse_message(desc, pr_num)
        if component is not None:
            note_in_desc = True
            component = process_component(component)
            history_notes.setdefault(component, []).append(note)
    # if description has no history notes, parse PR title/commit message
    # otherwise should skip PR title/commit message
    if not note_in_desc:
        component, note = parse_message(lines[0], pr_num)
        if component is not None:
            component = process_component(component)
            history_notes.setdefault(component, []).append(note)


def process_component(component):
    key = component.lower().replace(' ', '')
    if key in component_dict:
        if ' ' not in component_dict[key] and ' ' in component:
            component_dict[key] = component
        else:
            component = component_dict[key]
    return component


def parse_message(message: str, pr_num: str = None) -> (str, str):
    # do not include template
    if message.startswith('[Component Name'):
        return None, None
    m = re.search(r'^\[(.+?)\](.+)$', message)
    if m is not None:
        component = m.group(1)
        note = m.group(2).strip()
        if not pr_number_appended(note) and pr_num:
            note = '{} (#{})'.format(note, pr_num)
        note = re.sub('BREAKING CHANGE:',
                      '[BREAKING CHANGE]',
                      note,
                      flags=re.IGNORECASE)
        note = re.sub(r"^'(az .*?)':", r"`\1`:", note)
        note = re.sub(r"^(az .*?):", r"`\1`:", note)
        if not note.startswith('az') and not ':' in note:
            note = note[0].capitalize() + note[1:]
        if note.endswith('.'):
            note = note[:-1]
        return component, note
    return None, None


def pr_number_appended(line):
    m = re.search(r' \(#[0-9]+\)$', line)
    return m is not None


if __name__ == "__main__":
    generate_history_notes()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long, too-many-lines

import argparse
import unittest
try:
    import unittest.mock as mock
except ImportError:
    import mock

from knack.util import CLIError

from azure.cli.core.profiles import ResourceType

from azure.cli.command_modules.vm._validators import (_validate_vm_vmss_create_vnet,
                                                      _validate_vmss_create_subnet,
                                                      _validate_vm_create_storage_account,
                                                      _validate_vm_vmss_create_auth,
                                                      _validate_vm_create_storage_profile,
                                                      _validate_vmss_create_load_balancer_or_app_gateway)


def _get_test_cmd():
    from azure.cli.core.mock import DummyCli
    from azure.cli.core import AzCommandsLoader
    from azure.cli.core.commands import AzCliCommand
    cli_ctx = DummyCli()
    loader = AzCommandsLoader(cli_ctx, resource_type=ResourceType.MGMT_COMPUTE)
    cmd = AzCliCommand(loader, 'test', None)
    cmd.command_kwargs = {'resource_type': ResourceType.MGMT_COMPUTE}
    cmd.cli_ctx = cli_ctx
    return cmd


def _mock_resource_client(cli_ctx, client_type, **kwargs):
    client = mock.MagicMock()
    if client_type is ResourceType.MGMT_NETWORK:
        def _mock_list(rg):

            def _get_mock_vnet(name, rg, location):
                vnet = mock.MagicMock()
                vnet.name = name
                vnet.rg = rg
                vnet.location = location
                subnet = mock.MagicMock()
                subnet.name = '{}subnet'.format(name)
                vnet.subnets = [subnet]
                return vnet
            all_mocks = [
                _get_mock_vnet('vnet1', 'rg1', 'eastus'),
                _get_mock_vnet('vnet2', 'rg1', 'westus'),
                _get_mock_vnet('vnet3', 'rg2', 'westus'),
                _get_mock_vnet('vnet4', 'rg2', 'eastus')
            ]
            return [x for x in all_mocks if x.rg == rg]
        client.virtual_networks.list = _mock_list
    elif client_type is ResourceType.MGMT_RESOURCE_RESOURCES:
        def _mock_get(rg):
            def _get_mock_rg(name, location):
                mock_rg = mock.MagicMock()
                mock_rg.location = location
                mock_rg.name = name
                return mock_rg
            all_mocks = [
                _get_mock_rg('rg1', 'westus'),
                _get_mock_rg('rg2', 'eastus')
            ]
            return next((x for x in all_mocks if x.name == rg), _get_mock_rg(rg, 'unknown'))
        client.resource_groups.get = _mock_get
    elif client_type is ResourceType.MGMT_STORAGE:
        def _mock_list_by_resource_group(rg):
            def _get_mock_sa(name, rg, location, tier):
                mock_sa = mock.MagicMock()
                mock_sa.name = name
                mock_sa.resource_group = rg
                mock_sa.location = location
                mock_sa.sku.tier = tier
                return mock_sa
            all_mocks = [
                _get_mock_sa('sa1', 'rg1', 'eastus', 'Standard'),
                _get_mock_sa('sa2', 'rg1', 'eastus', 'Premium'),
                _get_mock_sa('sa3', 'rg1', 'westus', 'Standard'),
                _get_mock_sa('sa4', 'rg1', 'westus', 'Premium')
            ]
            return list(x for x in all_mocks if x.resource_group == rg)
        client.storage_accounts.list_by_resource_group = _mock_list_by_resource_group
    return client


def _mock_network_client_with_existing_subnet(*_, **kwargs):
    client = mock.MagicMock()

    def _mock_list(rg):
        def _get_mock_vnet(name, rg, location):
            vnet = mock.MagicMock()
            vnet.name = name
            vnet.rg = rg
            vnet.location = location
            subnet = mock.MagicMock()
            subnet.name = '{}subnet'.format(name)
            subnet.address_prefix = '10.0.0.0/24'
            vnet.subnets = [subnet]
            return vnet
        all_mocks = [
            _get_mock_vnet('vnet1', 'rg1', 'eastus'),
            _get_mock_vnet('vnet2', 'rg1', 'westus'),
        ]
        return [x for x in all_mocks if x.rg == rg]
    client.virtual_networks.list = _mock_list
    return client


class TestVMCreateDefaultVnet(unittest.TestCase):

    def _set_ns(self, rg, location=None):
        self.ns.resource_group_name = rg
        self.ns.location = location

    def setUp(self):
        ns = argparse.Namespace()
        ns.resource_group_name = None
        ns.subnet = None
        ns.vnet_name = None
        ns.vnet_type = None
        ns.location = None
        self.ns = ns

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_resource_client)
    def test_no_matching_vnet(self):
        self._set_ns('emptyrg', 'eastus')
        _validate_vm_vmss_create_vnet(_get_test_cmd(), self.ns)
        self.assertIsNone(self.ns.vnet_name)
        self.assertIsNone(self.ns.subnet)
        self.assertEqual(self.ns.vnet_type, 'new')

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_resource_client)
    def test_matching_vnet_specified_location(self):
        self._set_ns('rg1', 'eastus')
        _validate_vm_vmss_create_vnet(_get_test_cmd(), self.ns)
        self.assertEqual(self.ns.vnet_name, 'vnet1')
        self.assertEqual(self.ns.subnet, 'vnet1subnet')
        self.assertEqual(self.ns.vnet_type, 'existing')


class TestVMSSCreateDefaultVnet(unittest.TestCase):

    @staticmethod
    def _set_ns(rg, location=None):
        ns = argparse.Namespace()
        ns.resource_group_name = rg
        ns.location = location
        ns.subnet = None
        ns.vnet_name = None
        ns.vnet_type = None
        ns.disable_overprovision = None
        return ns

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_network_client_with_existing_subnet)
    def test_matching_vnet_subnet_size_matching(self):
        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.instance_count = 5
        _validate_vm_vmss_create_vnet(_get_test_cmd(), ns, for_scale_set=True)
        self.assertEqual(ns.vnet_name, 'vnet1')
        self.assertEqual(ns.subnet, 'vnet1subnet')
        self.assertEqual(ns.vnet_type, 'existing')

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_network_client_with_existing_subnet)
    def test_matching_vnet_no_subnet_size_matching(self):
        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.instance_count = 1000
        _validate_vm_vmss_create_vnet(_get_test_cmd(), ns, for_scale_set=True)
        self.assertIsNone(ns.vnet_name)
        self.assertIsNone(ns.subnet)
        self.assertEqual(ns.vnet_type, 'new')

        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.instance_count = 255
        _validate_vm_vmss_create_vnet(_get_test_cmd(), ns, for_scale_set=True)
        self.assertEqual(ns.vnet_type, 'new')

    def test_new_subnet_size_for_big_vmss_with_over_provision(self):
        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.vnet_type = 'new'
        ns.vnet_address_prefix = '10.0.0.0/16'
        ns.subnet_address_prefix = None
        ns.app_gateway_type = 'new'
        ns.app_gateway_subnet_address_prefix = '10.0.1.0/22'
        ns.instance_count = 1000
        # with over-provision, we has subnet size bigger than the capacity
        _validate_vmss_create_subnet(ns)
        self.assertEqual('10.0.0.0/21', ns.subnet_address_prefix)

    def test_new_subnet_size_for_big_vmss_without_over_provision(self):
        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.vnet_type = 'new'
        ns.vnet_address_prefix = '10.0.0.0/16'
        ns.subnet_address_prefix = None
        ns.app_gateway_type = 'new'
        ns.app_gateway_subnet_address_prefix = '10.0.1.0/22'
        ns.instance_count = 1000
        ns.disable_overprovision = True

        # w/o over-provision, we set subnet size just based on the capacity
        _validate_vmss_create_subnet(ns)
        self.assertEqual('10.0.0.0/22', ns.subnet_address_prefix)

    def test_new_subnet_size_for_small_vmss(self):
        ns = TestVMSSCreateDefaultVnet._set_ns('rg1', 'eastus')
        ns.vnet_type = 'new'
        ns.vnet_address_prefix = '10.0.0.0/16'
        ns.subnet_address_prefix = None
        ns.app_gateway_type = None
        ns.app_gateway_subnet_address_prefix = None
        ns.instance_count = 2
        _validate_vmss_create_subnet(ns)
        self.assertEqual('10.0.0.0/24', ns.subnet_address_prefix)


class TestVMCreateDefaultStorageAccount(unittest.TestCase):
    def __init__(self, methodName):
        super(TestVMCreateDefaultStorageAccount, self).__init__(methodName)
        self.ns = None

    def _set_ns(self, rg, location=None, tier='Standard'):
        ns = argparse.Namespace()
        ns.resource_group_name = rg
        ns.location = location
        ns.storage_sku = [tier]
        ns.storage_account = None
        ns.storage_account_type = None
        self.ns = ns

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_resource_client)
    def test_no_matching_storage_account(self):
        self._set_ns('emptyrg', 'eastus')
        _validate_vm_create_storage_account(_get_test_cmd(), self.ns)
        self.assertIsNone(self.ns.storage_account)
        self.assertEqual(self.ns.storage_account_type, 'new')

    @mock.patch('azure.cli.core.commands.client_factory.get_mgmt_service_client', _mock_resource_client)
    def test_matching_storage_account_specified_location(self):
        self._set_ns('rg1', 'eastus')
        _validate_vm_create_storage_account(_get_test_cmd(), self.ns)
        self.assertEqual(self.ns.storage_account, 'sa1')
        self.assertEqual(self.ns.storage_account_type, 'existing')

        self._set_ns('rg1', 'eastus', 'Premium')
        _validate_vm_create_storage_account(_get_test_cmd(), self.ns)
        self.assertEqual(self.ns.storage_account, 'sa2')
        self.assertEqual(self.ns.storage_account_type, 'existing')


class TestVMDefaultAuthType(unittest.TestCase):

    @staticmethod
    def _set_ns():
        ns = argparse.Namespace()
        ns.storage_profile = None
        ns.ssh_key_value = None
        ns.ssh_dest_key_path = None
        ns.admin_password = None
        ns.generate_ssh_keys = None
        return ns

    def test_default_windows(self):
        ns = TestVMDefaultAuthType._set_ns()
        ns.os_type = "WindowS"
        ns.authentication_type = None
        ns.admin_username = 'user12345'
        ns.admin_password = 'verySecret123'
        _validate_vm_vmss_create_auth(ns)
        self.assertEqual(ns.authentication_type, 'password')

    def test_default_linux(self):
        ns = TestVMDefaultAuthType._set_ns()
        ns.os_type = "LINux"
        ns.authentication_type = None
        test_user = 'user12345'
        ns.admin_username = test_user
        ns.admin_password = None
        ns.ssh_key_value = ['ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAACAQCbIg1guRHbI0lV11wWDt1r2cUdcNd27CJsg+SfgC7miZeubtwUhbsPdhMQsfDyhOWHq1+ZL0M+nJZV63d/1dhmhtgyOqejUwrPlzKhydsbrsdUor+JmNJDdW01v7BXHyuymT8G4s09jCasNOwiufbP/qp72ruu0bIA1nySsvlf9pCQAuFkAnVnf/rFhUlOkhtRpwcq8SUNY2zRHR/EKb/4NWY1JzR4sa3q2fWIJdrrX0DvLoa5g9bIEd4Df79ba7v+yiUBOS0zT2ll+z4g9izHK3EO5d8hL4jYxcjKs+wcslSYRWrascfscLgMlMGh0CdKeNTDjHpGPncaf3Z+FwwwjWeuiNBxv7bJo13/8B/098KlVDl4GZqsoBCEjPyJfV6hO0y/LkRGkk7oHWKgeWAfKtfLItRp00eZ4fcJNK9kCaSMmEugoZWcI7NGbZXzqFWqbpRI7NcDP9+WIQ+i9U5vqWsqd/zng4kbuAJ6UuKqIzB0upYrLShfQE3SAck8oaLhJqqq56VfDuASNpJKidV+zq27HfSBmbXnkR/5AK337dc3MXKJypoK/QPMLKUAP5XLPbs+NddJQV7EZXd29DLgp+fRIg3edpKdO7ZErWhv7d+3Kws+e1Y+ypmR2WIVSwVyBEUfgv2C8Ts9gnTF4pNcEY/S2aBicz5Ew2+jdyGNQQ== test@example.com\n']
        _validate_vm_vmss_create_auth(ns)
        self.assertEqual(ns.authentication_type, 'ssh')
        self.assertEqual(ns.ssh_dest_key_path, '/home/{}/.ssh/authorized_keys'.format(test_user))

    def test_linux_with_password(self):
        ns = TestVMDefaultAuthType._set_ns()
        ns.os_type = "LINux"
        ns.authentication_type = 'password'
        ns.admin_username = 'user12345'
        ns.admin_password = 'verySecret!!!'
        _validate_vm_vmss_create_auth(ns)
        # still has 'password'
        self.assertEqual(ns.authentication_type, 'password')

        # throw when conflict with ssh key value
        ns.ssh_key_value = ['junk but does not matter']
        with self.assertRaises(CLIError) as context:
            _validate_vm_vmss_create_auth(ns)
            self.assertTrue("SSH key cannot be used with password authentication type." in str(context.exception))


class TestVMImageDefaults(unittest.TestCase):
    @mock.patch('azure.cli.command_modules.vm._validators._compute_client_factory', autospec=True)
    def test_vm_validator_retrieve_image_info_cross_subscription(self, factory_mock):
        ns = argparse.Namespace()
        cmd = mock.MagicMock()

        data_disk = mock.MagicMock()
        data_disk.storage_account_type = 'Standard_LRS'
        data_disk.lun = 0
        image_info = mock.MagicMock()
        client_mock = mock.MagicMock()
        image_info.storage_profile.os_disk.os_type.value = 'someOS'
        image_info.storage_profile.data_disks = [data_disk]
        client_mock.images.get.return_value = image_info
        factory_mock.return_value = client_mock

        ns.image = '/subscriptions/0b1f6471-1bf0-4dda-aec3-xxxxxxxxxxxx/resourceGroups/foo/providers/Microsoft.Compute/images/bar'
        ns.admin_username = 'admin123'
        ns.admin_password = 'verySecret!'
        ns.storage_sku = ['Premium_LRS']
        ns.ultra_ssd_enabled = None
        ns.os_caching, ns.data_caching = None, None
        ns.os_type, ns.attach_os_disk, ns.storage_account, ns.storage_container_name, ns.use_unmanaged_disk, ns.data_disk_sizes_gb = None, None, None, None, False, None
        ns.size = 'Standard_DS1_v2'
        _validate_vm_create_storage_profile(cmd, ns, False)

        self.assertEqual(ns.os_type.value, 'someOS')
        self.assertTrue(0 in ns.disk_info)


class TestVMSSDefaults(unittest.TestCase):
    @classmethod
    def _set_up_ns(cls, ns):
        ns.single_placement_group, ns.zones, ns.platform_fault_domain_count, ns.instance_count = None, None, None, 2

    def test_vmss_default_std_lb(self):
        cmd = mock.MagicMock()
        lb_sku_mock = mock.MagicMock()
        lb_sku_mock.standard.value = 'Standard'
        lb_sku_mock.basic.value = 'Basic'
        cmd.get_models.return_value = lb_sku_mock

        # default to standard when single-placement-group is off
        ns = argparse.Namespace()
        ns.load_balancer, ns.application_gateway = None, None
        ns.app_gateway_subnet_address_prefix, ns.application_gateway = None, None
        ns.app_gateway_sku, ns.app_gateway_capacity = None, None
        ns.load_balancer_sku = None
        ns.single_placement_group = False
        _validate_vmss_create_load_balancer_or_app_gateway(cmd, ns)
        self.assertEqual(ns.load_balancer_sku, 'Standard')

        # error on conflicts
        ns = argparse.Namespace()
        ns.load_balancer, ns.application_gateway = None, None
        ns.app_gateway_subnet_address_prefix, ns.application_gateway = None, None
        ns.app_gateway_sku, ns.app_gateway_capacity = None, None
        ns.load_balancer_sku = 'Basic'
        ns.single_placement_group = False
        ns.zones = '1'
        self.assertRaises(CLIError, _validate_vmss_create_load_balancer_or_app_gateway, cmd, ns)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from knack.util import CLIError

logger = get_logger(__name__)

DEFAULT_INDEX_URL = "https://aka.ms/azure-cli-extension-index-v1"

ERR_TMPL_EXT_INDEX = 'Unable to get extension index.\n'
ERR_TMPL_NON_200 = '{}Server returned status code {{}} for {{}}'.format(ERR_TMPL_EXT_INDEX)
ERR_TMPL_NO_NETWORK = '{}Please ensure you have network connection. Error detail: {{}}'.format(ERR_TMPL_EXT_INDEX)
ERR_TMPL_BAD_JSON = '{}Response body does not contain valid json. Error detail: {{}}'.format(ERR_TMPL_EXT_INDEX)

ERR_UNABLE_TO_GET_EXTENSIONS = 'Unable to get extensions from index. Improper index format.'
TRIES = 3


# pylint: disable=inconsistent-return-statements
def get_index(index_url=None):
    import requests
    from azure.cli.core.util import should_disable_connection_verify
    index_url = index_url or DEFAULT_INDEX_URL

    for try_number in range(TRIES):
        try:
            response = requests.get(index_url, verify=(not should_disable_connection_verify()))
            if response.status_code == 200:
                return response.json()
            msg = ERR_TMPL_NON_200.format(response.status_code, index_url)
            raise CLIError(msg)
        except (requests.exceptions.ConnectionError, requests.exceptions.HTTPError) as err:
            msg = ERR_TMPL_NO_NETWORK.format(str(err))
            raise CLIError(msg)
        except ValueError as err:
            # Indicates that url is not redirecting properly to intended index url, we stop retrying after TRIES calls
            if try_number == TRIES - 1:
                msg = ERR_TMPL_BAD_JSON.format(str(err))
                raise CLIError(msg)
            import time
            time.sleep(0.5)
            continue


def get_index_extensions(index_url=None):
    index = get_index(index_url=index_url)
    extensions = index.get('extensions')
    if extensions is None:
        logger.warning(ERR_UNABLE_TO_GET_EXTENSIONS)
    return extensions



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
from knack.util import CLIError
from knack.log import get_logger

logger = get_logger(__name__)


def validate_headers(namespace):
    """Extracts multiple space-separated headers in key[=value] format. """
    if isinstance(namespace.headers, list):
        headers_dict = {}
        for item in namespace.headers:
            headers_dict.update(validate_header(item))
        namespace.headers = headers_dict


def validate_header(string):
    """Extracts a single header in key[=value] format. """
    result = {}
    if string:
        comps = string.split('=', 1)
        result = {comps[0]: comps[1]} if len(comps) > 1 else {string: ''}
    return result


def validate_arg(namespace):
    if isinstance(namespace.arg, list):
        arguments_list = []
        for item in namespace.arg:
            arguments_list.append(validate_task_argument(item, False))
        namespace.arg = arguments_list


def validate_secret_arg(namespace):
    if isinstance(namespace.secret_arg, list):
        secret_arguments_list = []
        for item in namespace.secret_arg:
            secret_arguments_list.append(validate_task_argument(item, True))
        namespace.secret_arg = secret_arguments_list


def validate_set(namespace):
    if isinstance(namespace.set_value, list):
        set_list = []
        for item in namespace.set_value:
            set_list.append(validate_task_value(item, False))
        namespace.set_value = set_list


def validate_set_secret(namespace):
    if isinstance(namespace.set_secret, list):
        set_secret_list = []
        for item in namespace.set_secret:
            set_secret_list.append(validate_task_value(item, True))
        namespace.set_secret = set_secret_list


def validate_task_value(string, is_secret):
    """Extracts a single SetValue in key[=value] format. """
    if string:
        comps = string.split('=', 1)
        if len(comps) > 1:
            return {'type': 'SetValue', 'name': comps[0], 'value': comps[1], 'isSecret': is_secret}
        return {'type': 'SetValue', 'name': comps[0], 'value': '', 'isSecret': is_secret}
    return None


def validate_task_argument(string, is_secret):
    """Extracts a single argument in key[=value] format. """
    if string:
        comps = string.split('=', 1)
        if len(comps) > 1:
            return {'type': 'Argument', 'name': comps[0], 'value': comps[1], 'isSecret': is_secret}
        # If no value, check if the argument exists as an environment variable
        local_value = os.environ.get(comps[0])
        if local_value is not None:
            return {'type': 'Argument', 'name': comps[0], 'value': local_value, 'isSecret': is_secret}
        return {'type': 'Argument', 'name': comps[0], 'value': '', 'isSecret': is_secret}
    return None


def validate_retention_days(namespace):
    days = namespace.days
    if days and (days < 0 or days > 365):
        raise CLIError("Invalid value for days: should be from 0 to 365")


def validate_registry_name(cmd, namespace):
    """Omit login server endpoint suffix."""
    registry = namespace.registry_name
    suffixes = cmd.cli_ctx.cloud.suffixes
    # Some clouds do not define 'acr_login_server_endpoint' (e.g. AzureGermanCloud)
    if registry and hasattr(suffixes, 'acr_login_server_endpoint'):
        acr_suffix = suffixes.acr_login_server_endpoint
        pos = registry.find(acr_suffix)
        if pos > 0:
            logger.warning("The login server endpoint suffix '%s' is automatically omitted.", acr_suffix)
            namespace.registry_name = registry[:pos]


def validate_expiration_time(namespace):
    import datetime
    DATE_TIME_FORMAT = '%Y-%m-%dT%H:%M:%SZ'
    if namespace.expiration:
        try:
            namespace.expiration = datetime.datetime.strptime(namespace.expiration, DATE_TIME_FORMAT)
        except ValueError:
            raise CLIError("Input '{}' is not valid datetime. Valid example: 2025-12-31T12:59:59Z".format(
                namespace.expiration))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer
import json


class TestDiagnosticSettingsSubscriptionScenarios(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_diagnostic_settings_subscription(self, resource_group, storage_account):
        self.kwargs.update({
            'name': self.create_random_name('clitest', 20),
            'ws': self.create_random_name('cliws', 20)
        })
        self.kwargs['storage'] = self.cmd('storage account show -n {sa} -g {rg} --query id -otsv').output.strip()
        self.kwargs['log_config'] = json.dumps([
            {
                "category": "Security",
                "enabled": True,
            },
            {
                "category": "Administrative",
                "enabled": True,
            },
            {
                "category": "ServiceHealth",
                "enabled": True,
            },
            {
                "category": "Alert",
                "enabled": True,
            },
            {
                "category": "Recommendation",
                "enabled": True,
            },
            {
                "category": "Policy",
                "enabled": True,
            },
            {
                "category": "Autoscale",
                "enabled": True,
            },
            {
                "category": "ResourceHealth",
                "enabled": True,
            }
        ])
        diagns = self.cmd("monitor diagnostic-settings subscription list").get_output_in_json()['value']
        for diagn in diagns:
            name = diagn['name']
            self.cmd("monitor diagnostic-settings subscription delete --name {} -y".format(name))
        self.cmd("monitor diagnostic-settings subscription create -l southcentralus --name {name} --storage-account {storage} "
                 "--logs \'{log_config}\'",
                 checks=[
                     self.check('storageAccountId', '{storage}'),
                     self.check('serviceBusRuleId', None),
                 ])
        self.cmd("monitor diagnostic-settings subscription show --name {name}", checks=[
            self.check('storageAccountId', '{storage}'),
            self.check('serviceBusRuleId', None),
        ])
        self.cmd("monitor diagnostic-settings subscription list", checks=[
            self.check('length(@)', 1)
        ])
        self.kwargs['ws_id'] = self.cmd('monitor log-analytics workspace create -n {ws} -g {rg} --query id -otsv').output.strip()
        self.cmd('monitor diagnostic-settings subscription update --name {name} --workspace {ws_id}', checks=[
            self.check('storageAccountId', '{storage}'),
            self.check('workspaceId', '{ws_id}')
        ])
        self.cmd("monitor diagnostic-settings subscription delete --name {name} -y")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from time import sleep
import unittest
from azure.cli.testsdk import ScenarioTest, JMESPathCheck, ResourceGroupPreparer, record_only
from azure.cli.command_modules.resource.custom import _parse_lock_id


class ResourceLockTests(ScenarioTest):
    def test_list_locks(self):
        # just make sure this doesn't throw
        self.cmd('az lock list').get_output_in_json()

    @record_only()
    def test_generic_subscription_locks(self):
        for lock_type in ['ReadOnly', 'CanNotDelete']:
            lock_name = self.create_random_name('cli-test-lock', 48)
            lock = self.cmd('az lock create -n {} --lock-type {}'.format(lock_name, lock_type)).get_output_in_json()
            lock_id = lock.get('id')
            self._sleep_for_lock_operation()

            locks_list = self.cmd('az lock list').get_output_in_json()
            self.assertTrue(locks_list)
            self.assertIn(lock_name, [lock['name'] for lock in locks_list])

            lock = self.cmd('az lock show -n {}'.format(lock_name)).get_output_in_json()
            lock_from_id = self.cmd('az lock show --ids {}'.format(lock_id)).get_output_in_json()

            self.assertEqual(lock.get('name', None), lock_name)
            self.assertEqual(lock_from_id.get('name', None), lock_name)
            self.assertEqual(lock.get('level', None), lock_type)

            notes = self.create_random_name('notes', 20)
            new_lvl = 'ReadOnly' if lock_type == 'CanNotDelete' else 'CanNotDelete'
            lock = self.cmd('az lock update -n {} --notes {} --lock-type {}'
                            .format(lock_name, notes, new_lvl)).get_output_in_json()
            self.assertEqual(lock.get('notes', None), notes)
            self.assertEqual(lock.get('level', None), new_lvl)

            lock = self.cmd('az lock update --ids {} --lock-type {}'
                            .format(lock_id, lock_type)).get_output_in_json()
            self.assertEqual(lock.get('level', None), lock_type)

            self.cmd('az lock delete -n {}'.format(lock_name))
            self._sleep_for_lock_operation()

    @ResourceGroupPreparer(name_prefix='cli_test_readonly_resource_group_lock')
    def test_readonly_resource_group_lock(self, resource_group):
        self._lock_operation_with_resource_group('ReadOnly', resource_group)

    @ResourceGroupPreparer(name_prefix='cli_test_cannotdelete_resource_group_lock')
    def test_cannotdelete_resource_group_lock(self, resource_group):
        self._lock_operation_with_resource_group('CanNotDelete', resource_group)

    @ResourceGroupPreparer(name_prefix='cli_test_readonly_resource_lock')
    def test_readonly_resource_lock(self, resource_group):
        self._lock_operation_with_resource('ReadOnly', resource_group)

    @ResourceGroupPreparer(name_prefix='cli_test_cannotdelete_resource_lock')
    def test_cannotdelete_resource_lock(self, resource_group):
        self._lock_operation_with_resource('CanNotDelete', resource_group)

    def _lock_operation_with_resource_group(self, lock_type, resource_group):

        self.kwargs = {
            'rg': resource_group,
            'type': lock_type,
            'lock': self.create_random_name('cli-test-lock', 48)
        }

        self.cmd('az lock create -n {lock} -g {rg} --lock-type {type}')
        self._sleep_for_lock_operation()

        self.cmd('az lock show -g {rg} -n {lock}', checks=[
            self.check('name', '{lock}'),
            self.check('level', '{type}')
        ])

        locks_list = self.cmd("az lock list -g {rg} --query '[].name'").get_output_in_json()
        self.assertTrue(locks_list)
        self.assertIn(self.kwargs['lock'], locks_list)

        self.kwargs.update({
            'notes': self.create_random_name('notes', 20),
            'new_lvl': 'ReadOnly' if lock_type == 'CanNotDelete' else 'CanNotDelete'
        })
        self.cmd('az lock update -n {lock} -g {rg} --notes {notes} --lock-type {new_lvl}', checks=[
            self.check('notes', '{notes}'),
            self.check('level', '{new_lvl}')
        ]).get_output_in_json()

        self.cmd('az lock delete -g {rg} -n {lock}')
        self._sleep_for_lock_operation()

    def _lock_operation_with_resource(self, lock_type, resource_group):
        rsrc_name = self.create_random_name('cli.lock.rsrc', 30)
        rsrc_type = 'Microsoft.Network/virtualNetworks'
        lock_name = self.create_random_name('cli-test-lock', 74)

        self.cmd('az network vnet create -n {} -g {}'.format(rsrc_name, resource_group))
        self.cmd('az lock create -n {} -g {} --resource-type {} --resource-name {} --lock-type {}'
                 .format(lock_name, resource_group, rsrc_type, rsrc_name, lock_type))
        self._sleep_for_lock_operation()

        self.cmd('az lock show --name {} -g {} --resource-type {} --resource-name {}'
                 .format(lock_name, resource_group, rsrc_type, rsrc_name)).assert_with_checks([
                     JMESPathCheck('name', lock_name),
                     JMESPathCheck('level', lock_type)])

        locks_list = self.cmd("az lock list --query '[].name' -ojson").get_output_in_json()
        self.assertTrue(locks_list)
        self.assertIn(lock_name, locks_list)

        notes = self.create_random_name('notes', 20)
        new_lvl = 'ReadOnly' if lock_type == 'CanNotDelete' else 'CanNotDelete'
        lock = self.cmd('az lock update -n {} -g {} --resource-type {} --resource-name {} --notes {} --lock-type {}'
                        .format(lock_name, resource_group, rsrc_type, rsrc_name, notes, new_lvl)).get_output_in_json()

        self.assertEqual(lock.get('notes', None), notes)
        self.assertEqual(lock.get('level', None), new_lvl)

        self.cmd('az lock delete --name {} -g {} --resource-name {} --resource-type {}'
                 .format(lock_name, resource_group, rsrc_name, rsrc_type))
        self._sleep_for_lock_operation()

    @ResourceGroupPreparer(name_prefix='cli_test_group_lock')
    def test_group_lock_commands(self, resource_group):

        self.kwargs = {
            'rg': resource_group,
            'lock': self.create_random_name('cli-test-lock', 48),
            'notes': self.create_random_name('notes', 20)
        }

        self.cmd('group lock create -n {lock} -g {rg} --lock-type CanNotDelete')
        self._sleep_for_lock_operation()

        self.cmd('group lock show -g {rg} -n {lock}', checks=[
            self.check('name', '{lock}'),
            self.check('level', 'CanNotDelete')
        ])

        locks_list = self.cmd("group lock list -g {rg} --query [].name").get_output_in_json()
        self.assertTrue(locks_list)
        self.assertIn(self.kwargs['lock'], locks_list)

        self.cmd('group lock update -n {lock} -g {rg} --notes {notes} --lock-type ReadOnly', checks=[
            self.check('notes', '{notes}'),
            self.check('level', 'ReadOnly')
        ])

        self.cmd('group lock delete -g {rg} -n {lock}')
        self._sleep_for_lock_operation()

    @ResourceGroupPreparer(name_prefix='cli_test_resource_lock')
    def test_resource_lock_commands(self, resource_group):
        rsrc_name = self.create_random_name('cli.lock.rsrc', 30)
        rsrc_type = 'Microsoft.Network/virtualNetworks'
        lock_name = self.create_random_name('cli-test-lock', 74)
        lock_type = 'CanNotDelete'

        self.cmd('network vnet create -n {} -g {}'.format(rsrc_name, resource_group))
        self.cmd('resource lock create -n {} -g {} --resource-type {} --resource-name {} --lock-type {}'
                 .format(lock_name, resource_group, rsrc_type, rsrc_name, lock_type))
        self._sleep_for_lock_operation()

        self.cmd('resource lock show --name {} -g {} --resource-type {} --resource-name {}'
                 .format(lock_name, resource_group, rsrc_type, rsrc_name)).assert_with_checks([
                     JMESPathCheck('name', lock_name),
                     JMESPathCheck('level', lock_type)])

        list_cmd = "resource lock list -g {} --resource-type {} --resource-name {} " \
                   "--query [].name -ojson".format(resource_group, rsrc_type, rsrc_name)
        locks_list = self.cmd(list_cmd).get_output_in_json()
        self.assertTrue(locks_list)
        self.assertIn(lock_name, locks_list)

        notes = self.create_random_name('notes', 20)
        lock = self.cmd('resource lock update -n {} -g {} --resource-type {} --resource-name {} --notes {} '
                        '--lock-type ReadOnly'
                        .format(lock_name, resource_group, rsrc_type, rsrc_name, notes)).get_output_in_json()

        self.assertEqual(lock.get('notes', None), notes)
        self.assertEqual(lock.get('level', None), 'ReadOnly')

        self.cmd('resource lock delete --name {} -g {} --resource-name {} --resource-type {}'
                 .format(lock_name, resource_group, rsrc_name, rsrc_type))
        self._sleep_for_lock_operation()

    @record_only()
    def test_subscription_locks(self):
        lock_name = self.create_random_name('cli-test-lock', 48)
        lock = self.cmd('az account lock create -n {} --lock-type CanNotDelete'.format(lock_name)).get_output_in_json()
        lock_id = lock.get('id')

        locks_list = self.cmd('az account lock list --query [].name').get_output_in_json()
        self.assertTrue(locks_list)
        self.assertIn(lock_name, locks_list)

        lock = self.cmd('az account lock show -n {}'.format(lock_name)).get_output_in_json()
        lock_from_id = self.cmd('az account lock show --ids {}'.format(lock_id)).get_output_in_json()

        self.assertEqual(lock.get('name', None), lock_name)
        self.assertEqual(lock_from_id.get('name', None), lock_name)
        self.assertEqual(lock.get('level', None), 'CanNotDelete')

        notes = self.create_random_name('notes', 20)
        lock = self.cmd('az account lock update -n {} --notes {} --lock-type {}'
                        .format(lock_name, notes, 'ReadOnly')).get_output_in_json()
        self.assertEqual(lock.get('notes', None), notes)
        self.assertEqual(lock.get('level', None), 'ReadOnly')

        lock = self.cmd('az account lock update --ids {} --lock-type {}'
                        .format(lock_id, 'CanNotDelete')).get_output_in_json()
        self.assertEqual(lock.get('level', None), 'CanNotDelete')

        self.cmd('az account lock delete -n {}'.format(lock_name))

    @ResourceGroupPreparer(name_prefix='cli_test_lock_commands_with_ids')
    def test_lock_commands_with_ids(self, resource_group):
        vnet_name = self.create_random_name('cli-lock-vnet', 30)
        subnet_name = self.create_random_name('cli-lock-subnet', 30)
        group_lock_name = self.create_random_name('cli-test-lock', 50)
        vnet_lock_name = self.create_random_name('cli-test-lock', 50)
        subnet_lock_name = self.create_random_name('cli-test-lock', 20)

        vnet = self.cmd('az network vnet create -n {} -g {}'.format(vnet_name, resource_group)).get_output_in_json()
        subnetaddress = vnet.get('newVNet').get('addressSpace').get('addressPrefixes')[0]
        self.cmd('az network vnet subnet create -n {} --address-prefix {} --vnet-name {} -g {}'
                 .format(subnet_name, subnetaddress, vnet_name, resource_group))

        locks = []
        locks.append(self.cmd('az lock create -n {} -g {} --lock-type CanNotDelete'
                              .format(group_lock_name, resource_group)).get_output_in_json())
        locks.append(self.cmd('az lock create -n {} -g {} --resource-type Microsoft.Network/virtualNetworks'
                              ' --resource-name {} --lock-type CanNotDelete'
                              .format(vnet_lock_name, resource_group, vnet_name)).get_output_in_json())
        locks.append(self.cmd('az lock create -n {} -g {} --resource-name {} --resource-type subnets '
                              '--namespace Microsoft.Network --parent virtualNetworks/{} --lock-type CanNotDelete'
                              .format(subnet_lock_name, resource_group, subnet_name, vnet_name)).get_output_in_json())
        self._sleep_for_lock_operation()

        space_delimited_ids = ' '.join([lock.get('id', None) for lock in locks])

        my_locks = self.cmd('az lock show --ids {} --query [].name'.format(space_delimited_ids)).get_output_in_json()
        self.assertTrue(len(my_locks) == 3)
        for lock in my_locks:
            self.assertIn(lock, [group_lock_name, vnet_lock_name, subnet_lock_name])

        my_locks = self.cmd('az lock update --ids {} --notes somenotes --lock-type ReadOnly'
                            .format(space_delimited_ids)).get_output_in_json()
        self.assertTrue(len(my_locks) == 3)
        for lock in my_locks:
            self.assertEqual(lock.get('notes', None), 'somenotes')
            self.assertEqual(lock.get('level', None), 'ReadOnly')

        self.cmd('az lock delete --ids {}'.format(space_delimited_ids))
        self._sleep_for_lock_operation()

        my_locks = self.cmd("az lock list -g {} -ojson".format(resource_group)).get_output_in_json()
        self.assertFalse(my_locks)

    @ResourceGroupPreparer(name_prefix='cli_test_lock_with_resource_id')
    def test_lock_with_resource_id(self, resource_group):
        vnet_name = self.create_random_name('cli-lock-vnet', 30)
        subnet_name = self.create_random_name('cli-lock-subnet', 30)
        vnet_lock_name = self.create_random_name('cli-test-lock', 50)
        subnet_lock_name = self.create_random_name('cli-test-lock', 20)

        vnet = self.cmd('network vnet create -n {} -g {}'.format(vnet_name, resource_group)).get_output_in_json()
        vnet_id = vnet.get('newVNet').get('id')
        subnetaddress = vnet.get('newVNet').get('addressSpace').get('addressPrefixes')[0]
        subnet_id = self.cmd('network vnet subnet create -n {} --address-prefix {} --vnet-name {} -g {}'
                             .format(subnet_name, subnetaddress,
                                     vnet_name, resource_group)).get_output_in_json().get('id')

        self.cmd('resource lock create -n {} --resource {} --lock-type CanNotDelete'.format(vnet_lock_name, vnet_id))
        self.cmd('lock create -n {} --resource {} --lock-type CanNotDelete'.format(subnet_lock_name, subnet_id))

        self.cmd('resource lock show --name {} --resource {}'
                 .format(vnet_lock_name, vnet_id)).assert_with_checks([JMESPathCheck('name', vnet_lock_name)])
        self.cmd('lock show --name {} --resource {}'
                 .format(subnet_lock_name, subnet_id)).assert_with_checks([JMESPathCheck('name', subnet_lock_name)])

        self.cmd('resource lock delete --name {} --resource {}'.format(vnet_lock_name, vnet_id))
        self.cmd('lock delete --name {} --resource {}'.format(subnet_lock_name, subnet_id))

        self._sleep_for_lock_operation()

    def _sleep_for_lock_operation(self):
        if self.is_live:
            sleep(5)


class ParseIdTests(unittest.TestCase):
    def test_parsing_lock_ids(self):
        tests = [
            {
                'input': "/subscriptions/subId/providers/"
                         "Microsoft.Authorization/locks/sublock",
                'expected': {
                    'resource_group': None,
                    'resource_provider_namespace': None,
                    'parent_resource_path': None,
                    'resource_type': None,
                    'resource_name': None,
                    'lock_name': 'sublock'
                }
            },
            {
                'input': "/subscriptions/subId/resourceGroups/examplegroup/providers/"
                         "Microsoft.Authorization/locks/grouplock",
                'expected': {
                    'resource_group': 'examplegroup',
                    'resource_provider_namespace': None,
                    'parent_resource_path': None,
                    'resource_type': None,
                    'resource_name': None,
                    'lock_name': 'grouplock'
                }
            },
            {
                'input': "/subscriptions/subId/resourcegroups/mygroup/providers/"
                         "Microsoft.Network/virtualNetworks/myvnet/providers/"
                         "Microsoft.Authorization/locks/vnetlock",
                'expected': {
                    'resource_group': 'mygroup',
                    'resource_provider_namespace': 'Microsoft.Network',
                    'parent_resource_path': None,
                    'resource_type': 'virtualNetworks',
                    'resource_name': 'myvnet',
                    'lock_name': 'vnetlock'
                }
            },
            {
                'input': "/subscriptions/subId/resourceGroups/mygroup/providers/"
                         "Microsoft.Network/virtualNetworks/myvnet/subnets/subnet/providers/"
                         "Microsoft.Authorization/locks/subnetlock",
                'expected': {
                    'resource_group': 'mygroup',
                    'resource_provider_namespace': 'Microsoft.Network',
                    'parent_resource_path': 'virtualNetworks/myvnet',
                    'resource_type': 'subnets',
                    'resource_name': 'subnet',
                    'lock_name': 'subnetlock'
                }
            },
            {
                'input': "/subscriptions/subId/resourceGroups/mygroup/providers/"
                         "Microsoft.Provider1/resourceType1/name1/providers/"
                         "Microsoft.Provider2/resourceType2/name2/providers/"
                         "Microsoft.Authorization/locks/somelock",
                'expected': {
                    'resource_group': 'mygroup',
                    'resource_provider_namespace': 'Microsoft.Provider1',
                    'parent_resource_path': 'resourceType1/name1/providers/Microsoft.Provider2',
                    'resource_type': 'resourceType2',
                    'resource_name': 'name2',
                    'lock_name': 'somelock'
                }
            }
        ]
        for test in tests:
            kwargs = _parse_lock_id(test['input'])
            self.assertDictEqual(kwargs, test['expected'])

        fail_tests = [
            "/notsubscriptions/subId/providers/Microsoft.Authorization/locks/sublock",
            "/subscriptions/subId/notResourceGroups/examplegroup/providers/Microsoft.Authorization/locks/grouplock",
            "/subscriptions/subId/resourceGroups/examplegroup/providers/Microsoft.NotAuthorization/not_locks/grouplock",
            "/subscriptions/subId/resourcegroups/mygroup/Microsoft.Network/virtualNetworks/myvnet/providers/"
            "Microsoft.Authorization/locks/missingProvidersLock",
            "/subscriptions/subId/resourcegroups/mygroup/providers/Microsoft.Network/myvnet/providers/"
            "Microsoft.Authorization/locks/missingRsrcTypeLock",
            "/subscriptions/subId/providers/Microsoft.Network/virtualNetworks/myvnet/subnets/subnet/providers/"
            "Microsoft.Authorization/locks/missingRsrcGroupLock",
            "not_a_id_at_all"
        ]
        for test in fail_tests:
            with self.assertRaises(AttributeError):
                _parse_lock_id(test)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import (ScenarioTest, JMESPathCheck, ResourceGroupPreparer, StorageAccountPreparer)
from azure_devtools.scenario_tests import AllowLargeResponse


class StorageLegalHold(ScenarioTest):
    @AllowLargeResponse()
    @ResourceGroupPreparer()
    def test_legal_hold(self, resource_group):
        storage_account = self.create_random_name('clistorage', 20)
        self.cmd('storage account create -g {} -n {} --kind StorageV2'.format(
            resource_group, storage_account))
        container_name = 'container1'
        self.cmd('storage container create --account-name {} -n {}'.format(storage_account, container_name))

        self.cmd('storage container legal-hold show --account-name {} -c {} -g {}'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck("tags", [])])

        result = self.cmd('storage container legal-hold set --account-name {} -c {} -g {} --tags tag1 tag2'.format(
            storage_account, container_name, resource_group)).get_output_in_json()
        self.assertIn("tag1", result.get("tags"))
        self.assertIn("tag2", result.get("tags"))

        self.cmd('storage container legal-hold clear --account-name {} -c {} -g {} --tags tag1 tag2'.format(
            storage_account, container_name, resource_group), checks=[
                JMESPathCheck("tags", [])])



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from __future__ import print_function
from codecs import open
from setuptools import setup, find_packages

VERSION = "2.18.0"

# If we have source, validate that our version numbers match
# This should prevent uploading releases with mismatched versions.
try:
    with open('azure/cli/core/__init__.py', 'r', encoding='utf-8') as f:
        content = f.read()
except OSError:
    pass
else:
    import re
    import sys

    m = re.search(r'__version__\s*=\s*[\'"](.+?)[\'"]', content)
    if not m:
        print('Could not find __version__ in azure/cli/core/__init__.py')
        sys.exit(1)
    if m.group(1) != VERSION:
        print('Expected __version__ = "{}"; found "{}"'.format(VERSION, m.group(1)))
        sys.exit(1)

CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'License :: OSI Approved :: MIT License',
]

DEPENDENCIES = [
    'adal~=1.2.3',
    'argcomplete~=1.8',
    'azure-cli-telemetry==1.0.6.*',
    'colorama~=0.4.1',
    'humanfriendly>=4.7,<10.0',
    'jmespath',
    'knack==0.8.0rc2',
    'msal~=1.0.0',
    'msal-extensions~=0.1.3',
    'msrestazure>=0.6.3',
    'paramiko>=2.0.8,<3.0.0',
    'PyJWT',
    'pyopenssl>=17.1.0',  # https://github.com/pyca/pyopenssl/pull/612
    'requests~=2.22',
    'six~=1.12',
    'pkginfo>=1.5.0.1',
    'azure-mgmt-core==1.2.1',
    # Dependencies of the vendored subscription SDK
    # https://github.com/Azure/azure-sdk-for-python/blob/ab12b048ddf676fe0ccec16b2167117f0609700d/sdk/resources/azure-mgmt-resource/setup.py#L82-L86
    'msrest>=0.5.0',
    'azure-common~=1.1',
]

TESTS_REQUIRE = [
    'mock'
]

with open('README.rst', 'r', encoding='utf-8') as f:
    README = f.read()
with open('HISTORY.rst', 'r', encoding='utf-8') as f:
    HISTORY = f.read()

setup(
    name='azure-cli-core',
    version=VERSION,
    description='Microsoft Azure Command-Line Tools Core Module',
    long_description=README + '\n\n' + HISTORY,
    license='MIT',
    author='Microsoft Corporation',
    author_email='azpycli@microsoft.com',
    url='https://github.com/Azure/azure-cli',
    zip_safe=False,
    classifiers=CLASSIFIERS,
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests", "azure", "azure.cli"]),
    install_requires=DEPENDENCIES,
    python_requires='>=3.6.0',
    extras_require={
        "test": TESTS_REQUIRE,
    },
    tests_require=TESTS_REQUIRE,
    package_data={'azure.cli.core': ['auth_landing_pages/*.html']}
)
